@@ -1,25 +1,6 @@
 /* -*- mode: C++; c-basic-offset: 4; indent-tabs-mode: nil; -*- */
-/*
- * Copyright (c) 2013 Giovanni Campagna <scampa.giovanni@gmail.com>
- *
- * Permission is hereby granted, free of charge, to any person obtaining a copy
- * of this software and associated documentation files (the "Software"), to
- * deal in the Software without restriction, including without limitation the
- * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
- * sell copies of the Software, and to permit persons to whom the Software is
- * furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice shall be included in
- * all copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
- * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
- * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
- * IN THE SOFTWARE.
- */
+// SPDX-License-Identifier: MIT OR LGPL-2.0-or-later
+// SPDX-FileCopyrightText: 2013 Giovanni Campagna <scampa.giovanni@gmail.com>
 
 #include <config.h>
 
@@ -27,6 +8,9 @@
 #include <stdint.h>
 #include <string.h>
 
+#include <limits>
+#include <unordered_set>  // for unordered_set::erase(), insert()
+
 #include <ffi.h>
 #include <girepository.h>
 #include <glib.h>
@@ -42,6 +26,7 @@
 
 #include "gi/arg-cache.h"
 #include "gi/arg-inl.h"
+#include "gi/arg-types-inl.h"
 #include "gi/arg.h"
 #include "gi/boxed.h"
 #include "gi/foreign.h"
@@ -49,12 +34,15 @@
 #include "gi/fundamental.h"
 #include "gi/gerror.h"
 #include "gi/gtype.h"
+#include "gi/js-value-inl.h"
 #include "gi/object.h"
 #include "gi/param.h"
 #include "gi/union.h"
 #include "gi/value.h"
+#include "gi/wrapperutils.h"  // for GjsTypecheckNoThrow
 #include "gjs/byteArray.h"
 #include "gjs/jsapi-util.h"
+#include "util/log.h"
 
 enum ExpectedType {
     OBJECT,
@@ -67,36 +55,30 @@ static const char* expected_type_names[] = {"object", "function", "string"};
 static_assert(G_N_ELEMENTS(expected_type_names) == ExpectedType::LAST,
               "Names must match the values in ExpectedType");
 
-// The global entry point for any invocations of GDestroyNotify; look up the
-// callback through the user_data and then free it.
-static void gjs_destroy_notify_callback(void* data) {
-    auto* trampoline = static_cast<GjsCallbackTrampoline*>(data);
-
-    g_assert(trampoline);
-    gjs_callback_trampoline_unref(trampoline);
-}
-
 // A helper function to retrieve array lengths from a GIArgument (letting the
 // compiler generate good instructions in case of big endian machines)
 [[nodiscard]] static size_t gjs_g_argument_get_array_length(GITypeTag tag,
                                                             GIArgument* arg) {
-    if (tag == GI_TYPE_TAG_INT8)
-        return gjs_arg_get<int8_t>(arg);
-    if (tag == GI_TYPE_TAG_UINT8)
-        return gjs_arg_get<uint8_t>(arg);
-    if (tag == GI_TYPE_TAG_INT16)
-        return gjs_arg_get<int16_t>(arg);
-    if (tag == GI_TYPE_TAG_UINT16)
-        return gjs_arg_get<uint16_t>(arg);
-    if (tag == GI_TYPE_TAG_INT32)
-        return gjs_arg_get<int32_t>(arg);
-    if (tag == GI_TYPE_TAG_UINT32)
-        return gjs_arg_get<uint32_t>(arg);
-    if (tag == GI_TYPE_TAG_INT64)
-        return gjs_arg_get<int64_t>(arg);
-    if (tag == GI_TYPE_TAG_UINT64)
-        return gjs_arg_get<uint64_t>(arg);
-    g_assert_not_reached();
+    switch (tag) {
+        case GI_TYPE_TAG_INT8:
+            return gjs_arg_get<int8_t>(arg);
+        case GI_TYPE_TAG_UINT8:
+            return gjs_arg_get<uint8_t>(arg);
+        case GI_TYPE_TAG_INT16:
+            return gjs_arg_get<int16_t>(arg);
+        case GI_TYPE_TAG_UINT16:
+            return gjs_arg_get<uint16_t>(arg);
+        case GI_TYPE_TAG_INT32:
+            return gjs_arg_get<int32_t>(arg);
+        case GI_TYPE_TAG_UINT32:
+            return gjs_arg_get<uint32_t>(arg);
+        case GI_TYPE_TAG_INT64:
+            return gjs_arg_get<int64_t>(arg);
+        case GI_TYPE_TAG_UINT64:
+            return gjs_arg_get<uint64_t>(arg);
+        default:
+            g_assert_not_reached();
+    }
 }
 
 static void gjs_g_argument_set_array_length(GITypeTag tag, GIArgument* arg,
@@ -174,14 +156,6 @@ static bool report_gtype_mismatch(JSContext* cx, const char* arg_name,
     return false;
 }
 
-GJS_JSAPI_RETURN_CONVENTION
-static bool report_out_of_range(JSContext* cx, const char* arg_name,
-                                GITypeTag tag) {
-    gjs_throw(cx, "Argument %s: value is out of range for %s", arg_name,
-              g_type_tag_to_string(tag));
-    return false;
-}
-
 GJS_JSAPI_RETURN_CONVENTION
 static bool report_invalid_null(JSContext* cx, const char* arg_name) {
     gjs_throw(cx, "Argument %s may not be null", arg_name);
@@ -215,7 +189,7 @@ static bool gjs_marshal_generic_in_in(JSContext* cx, GjsArgumentCache* self,
                                    self->is_return_value()
                                        ? GJS_ARGUMENT_RETURN_VALUE
                                        : GJS_ARGUMENT_ARGUMENT,
-                                   self->transfer, self->nullable, arg);
+                                   self->transfer, self->flags, arg);
 }
 
 GJS_JSAPI_RETURN_CONVENTION
@@ -243,7 +217,7 @@ static bool gjs_marshal_explicit_array_in_in(JSContext* cx,
 
     if (!gjs_array_to_explicit_array(
             cx, value, &self->type_info, self->arg_name, GJS_ARGUMENT_ARGUMENT,
-            self->transfer, self->nullable, &data, &length))
+            self->transfer, self->flags, &data, &length))
         return false;
 
     uint8_t length_pos = self->contents.array.length_pos;
@@ -295,7 +269,7 @@ static bool gjs_marshal_callback_in(JSContext* cx, GjsArgumentCache* self,
     GjsCallbackTrampoline* trampoline;
     ffi_closure* closure;
 
-    if (value.isNull() && self->nullable) {
+    if (value.isNull() && (self->flags & GjsArgumentFlags::MAY_BE_NULL)) {
         closure = nullptr;
         trampoline = nullptr;
     } else {
@@ -322,24 +296,33 @@ static bool gjs_marshal_callback_in(JSContext* cx, GjsArgumentCache* self,
                 return false;
             }
 
-            priv->associate_closure(cx, trampoline->js_function);
+            priv->associate_closure(cx, trampoline->js_function());
         }
-        closure = trampoline->closure;
+        closure = trampoline->closure();
     }
 
     if (self->has_callback_destroy()) {
         uint8_t destroy_pos = self->contents.callback.destroy_pos;
-        gjs_arg_set(&state->in_cvalues[destroy_pos],
-                    trampoline ? gjs_destroy_notify_callback : nullptr);
+        GDestroyNotify destroy_notify = nullptr;
+        if (trampoline) {
+            /* Adding another reference and a DestroyNotify that unsets it */
+            gjs_callback_trampoline_ref(trampoline);
+            destroy_notify = [](void* data) {
+                g_assert(data);
+                gjs_callback_trampoline_unref(
+                    static_cast<GjsCallbackTrampoline*>(data));
+            };
+        }
+        gjs_arg_set(&state->in_cvalues[destroy_pos], destroy_notify);
     }
     if (self->has_callback_closure()) {
         uint8_t closure_pos = self->contents.callback.closure_pos;
         gjs_arg_set(&state->in_cvalues[closure_pos], trampoline);
     }
 
-    if (trampoline && self->contents.callback.scope != GI_SCOPE_TYPE_CALL) {
-        // Add an extra reference that will be cleared when collecting async
-        // calls, or when GDestroyNotify is called
+    if (trampoline && self->contents.callback.scope == GI_SCOPE_TYPE_ASYNC) {
+        // Add an extra reference that will be cleared when garbage collecting
+        // async calls
         gjs_callback_trampoline_ref(trampoline);
     }
     gjs_arg_set(arg, closure);
@@ -362,7 +345,7 @@ GJS_JSAPI_RETURN_CONVENTION
 static bool gjs_marshal_caller_allocates_in(JSContext*, GjsArgumentCache* self,
                                             GjsFunctionCallState* state,
                                             GIArgument* arg, JS::HandleValue) {
-    void* blob = g_slice_alloc0(self->contents.caller_allocates_size);
+    void* blob = g_malloc0(self->contents.caller_allocates_size);
     gjs_arg_set(arg, blob);
     gjs_arg_set(&state->out_cvalues[self->arg_pos], blob);
     return true;
@@ -383,79 +366,59 @@ static bool gjs_marshal_boolean_in_in(JSContext*, GjsArgumentCache*,
     return true;
 }
 
-// Type tags are alternated, signed / unsigned
-static int32_t min_max_ints[5][2] = {{G_MININT8, G_MAXINT8},
-                                     {0, G_MAXUINT8},
-                                     {G_MININT16, G_MAXINT16},
-                                     {0, G_MAXUINT16},
-                                     {G_MININT32, G_MAXINT32}};
+template <typename T>
+GJS_JSAPI_RETURN_CONVENTION inline static bool gjs_arg_set_from_js_value(
+    JSContext* cx, const JS::HandleValue& value, GArgument* arg,
+    GjsArgumentCache* self) {
+    bool out_of_range = false;
 
-[[nodiscard]] static inline bool value_in_range(int32_t number, GITypeTag tag) {
-    return (number >= min_max_ints[tag - GI_TYPE_TAG_INT8][0] &&
-            number <= min_max_ints[tag - GI_TYPE_TAG_INT8][1]);
+    if (!gjs_arg_set_from_js_value<T>(cx, value, arg, &out_of_range)) {
+        if (out_of_range) {
+            gjs_throw(cx, "Argument %s: value is out of range for %s",
+                      self->arg_name, Gjs::static_type_name<T>());
+        }
+
+        return false;
+    }
+
+    gjs_debug_marshal(GJS_DEBUG_GFUNCTION, "%s set to value %s (type %s)",
+                      GjsAutoChar(gjs_argument_display_name(
+                                      self->arg_name, GJS_ARGUMENT_ARGUMENT))
+                          .get(),
+                      std::to_string(gjs_arg_get<T>(arg)).c_str(),
+                      Gjs::static_type_name<T>());
+
+    return true;
 }
 
 GJS_JSAPI_RETURN_CONVENTION
-static bool gjs_marshal_integer_in_in(JSContext* cx, GjsArgumentCache* self,
+static bool gjs_marshal_numeric_in_in(JSContext* cx, GjsArgumentCache* self,
                                       GjsFunctionCallState*, GIArgument* arg,
                                       JS::HandleValue value) {
-    GITypeTag tag = self->contents.number.number_tag;
-
-    if (self->is_unsigned) {
-        uint32_t number;
-        if (!JS::ToUint32(cx, value, &number))
-            return false;
-
-        if (!value_in_range(number, tag))
-            return report_out_of_range(cx, self->arg_name, tag);
-
-        gjs_g_argument_set_array_length(tag, arg, number);
-    } else {
-        int32_t number;
-        if (!JS::ToInt32(cx, value, &number))
-            return false;
-
-        if (!value_in_range(number, tag))
-            return report_out_of_range(cx, self->arg_name, tag);
-
-        gjs_g_argument_set_array_length(tag, arg, number);
+    switch (self->contents.number.number_tag) {
+        case GI_TYPE_TAG_INT8:
+            return gjs_arg_set_from_js_value<int8_t>(cx, value, arg, self);
+        case GI_TYPE_TAG_UINT8:
+            return gjs_arg_set_from_js_value<uint8_t>(cx, value, arg, self);
+        case GI_TYPE_TAG_INT16:
+            return gjs_arg_set_from_js_value<int16_t>(cx, value, arg, self);
+        case GI_TYPE_TAG_UINT16:
+            return gjs_arg_set_from_js_value<uint16_t>(cx, value, arg, self);
+        case GI_TYPE_TAG_INT32:
+            return gjs_arg_set_from_js_value<int32_t>(cx, value, arg, self);
+        case GI_TYPE_TAG_DOUBLE:
+            return gjs_arg_set_from_js_value<double>(cx, value, arg, self);
+        case GI_TYPE_TAG_FLOAT:
+            return gjs_arg_set_from_js_value<float>(cx, value, arg, self);
+        case GI_TYPE_TAG_INT64:
+            return gjs_arg_set_from_js_value<int64_t>(cx, value, arg, self);
+        case GI_TYPE_TAG_UINT64:
+            return gjs_arg_set_from_js_value<uint64_t>(cx, value, arg, self);
+        case GI_TYPE_TAG_UINT32:
+            return gjs_arg_set_from_js_value<uint32_t>(cx, value, arg, self);
+        default:
+            g_assert_not_reached();
     }
-
-    return true;
-}
-
-GJS_JSAPI_RETURN_CONVENTION
-static bool gjs_marshal_number_in_in(JSContext* cx, GjsArgumentCache* self,
-                                     GjsFunctionCallState*, GIArgument* arg,
-                                     JS::HandleValue value) {
-    double v;
-    if (!JS::ToNumber(cx, value, &v))
-        return false;
-
-    GITypeTag tag = self->contents.number.number_tag;
-    if (tag == GI_TYPE_TAG_DOUBLE) {
-        gjs_arg_set(arg, v);
-    } else if (tag == GI_TYPE_TAG_FLOAT) {
-        if (v < -G_MAXFLOAT || v > G_MAXFLOAT)
-            return report_out_of_range(cx, self->arg_name, GI_TYPE_TAG_FLOAT);
-        gjs_arg_set<float>(arg, v);
-    } else if (tag == GI_TYPE_TAG_INT64) {
-        if (v < G_MININT64 || v > G_MAXINT64)
-            return report_out_of_range(cx, self->arg_name, GI_TYPE_TAG_INT64);
-        gjs_arg_set<int64_t>(arg, v);
-    } else if (tag == GI_TYPE_TAG_UINT64) {
-        if (v < 0 || v > G_MAXUINT64)
-            return report_out_of_range(cx, self->arg_name, GI_TYPE_TAG_UINT64);
-        gjs_arg_set<uint64_t>(arg, v);
-    } else if (tag == GI_TYPE_TAG_UINT32) {
-        if (v < 0 || v > G_MAXUINT32)
-            return report_out_of_range(cx, self->arg_name, GI_TYPE_TAG_UINT32);
-        gjs_arg_set<uint32_t>(arg, v);
-    } else {
-        g_assert_not_reached();
-    }
-
-    return true;
 }
 
 GJS_JSAPI_RETURN_CONVENTION
@@ -486,7 +449,7 @@ static bool gjs_marshal_gtype_in_in(JSContext* cx, GjsArgumentCache* self,
 
 // Common code for most types that are pointers on the C side
 bool GjsArgumentCache::handle_nullable(JSContext* cx, GIArgument* arg) {
-    if (!nullable)
+    if (!(flags & GjsArgumentFlags::MAY_BE_NULL))
         return report_invalid_null(cx, arg_name);
     gjs_arg_unset<void*>(arg);
     return true;
@@ -503,7 +466,7 @@ static bool gjs_marshal_string_in_in(JSContext* cx, GjsArgumentCache* self,
         return report_typeof_mismatch(cx, self->arg_name, value,
                                       ExpectedType::STRING);
 
-    if (self->contents.string_is_filename) {
+    if (self->flags & GjsArgumentFlags::FILENAME) {
         GjsAutoChar str;
         if (!gjs_string_to_filename(cx, value, &str))
             return false;
@@ -523,13 +486,13 @@ static bool gjs_marshal_enum_in_in(JSContext* cx, GjsArgumentCache* self,
                                    GjsFunctionCallState*, GIArgument* arg,
                                    JS::HandleValue value) {
     int64_t number;
-    if (!JS::ToInt64(cx, value, &number))
+    if (!Gjs::js_value_to_c(cx, value, &number))
         return false;
 
     // Unpack the values from their uint32_t bitfield. See note in
     // gjs_arg_cache_build_enum_bounds().
     int64_t min, max;
-    if (self->is_unsigned) {
+    if (self->flags & GjsArgumentFlags::UNSIGNED) {
         min = self->contents.enum_type.enum_min;
         max = self->contents.enum_type.enum_max;
     } else {
@@ -543,7 +506,7 @@ static bool gjs_marshal_enum_in_in(JSContext* cx, GjsArgumentCache* self,
         return false;
     }
 
-    if (self->is_unsigned)
+    if (self->flags & GjsArgumentFlags::UNSIGNED)
         gjs_arg_set<unsigned, GI_TYPE_TAG_INTERFACE>(arg, number);
     else
         gjs_arg_set<int, GI_TYPE_TAG_INTERFACE>(arg, number);
@@ -556,7 +519,7 @@ static bool gjs_marshal_flags_in_in(JSContext* cx, GjsArgumentCache* self,
                                     GjsFunctionCallState*, GIArgument* arg,
                                     JS::HandleValue value) {
     int64_t number;
-    if (!JS::ToInt64(cx, value, &number))
+    if (!Gjs::js_value_to_c(cx, value, &number))
         return false;
 
     if ((uint64_t(number) & self->contents.flags_mask) != uint64_t(number)) {
@@ -580,13 +543,27 @@ static bool gjs_marshal_foreign_in_in(JSContext* cx, GjsArgumentCache* self,
     self->contents.tmp_foreign_info = foreign_info;
     return gjs_struct_foreign_convert_to_g_argument(
         cx, value, foreign_info, self->arg_name, GJS_ARGUMENT_ARGUMENT,
-        self->transfer, self->nullable, arg);
+        self->transfer, self->flags, arg);
 }
 
 GJS_JSAPI_RETURN_CONVENTION
 static bool gjs_marshal_gvalue_in_in(JSContext* cx, GjsArgumentCache*,
-                                     GjsFunctionCallState*, GIArgument* arg,
-                                     JS::HandleValue value) {
+                                     GjsFunctionCallState* state,
+                                     GIArgument* arg, JS::HandleValue value) {
+    if (value.isObject()) {
+        JS::RootedObject obj(cx, &value.toObject());
+        GType gtype;
+
+        if (!gjs_gtype_get_actual_gtype(cx, obj, &gtype))
+            return false;
+
+        if (gtype == G_TYPE_VALUE) {
+            gjs_arg_set(arg, BoxedBase::to_c_ptr<GValue>(cx, obj));
+            state->ignore_release.insert(arg);
+            return true;
+        }
+    }
+
     GValue gvalue = G_VALUE_INIT;
 
     if (!gjs_value_to_g_value(cx, value, &gvalue))
@@ -937,11 +914,12 @@ static bool gjs_marshal_explicit_array_inout_release(
 }
 
 GJS_JSAPI_RETURN_CONVENTION
-static bool gjs_marshal_caller_allocates_release(
-    JSContext*, GjsArgumentCache* self, GjsFunctionCallState*,
-    GIArgument* in_arg, GIArgument* out_arg [[maybe_unused]]) {
-    g_slice_free1(self->contents.caller_allocates_size,
-                  gjs_arg_get<void*>(in_arg));
+static bool gjs_marshal_caller_allocates_release(JSContext*, GjsArgumentCache*,
+                                                 GjsFunctionCallState*,
+                                                 GIArgument* in_arg,
+                                                 GIArgument* out_arg
+                                                 [[maybe_unused]]) {
+    g_free(gjs_arg_steal<void*>(in_arg));
     return true;
 }
 
@@ -954,11 +932,11 @@ static bool gjs_marshal_callback_release(JSContext*, GjsArgumentCache*,
     if (!closure)
         return true;
 
-    auto trampoline = static_cast<GjsCallbackTrampoline*>(closure->user_data);
+    GjsAutoCallbackTrampoline trampoline =
+        static_cast<GjsCallbackTrampoline*>(closure->user_data);
     // CallbackTrampolines are refcounted because for notified/async closures
     // it is possible to destroy it while in call, and therefore we cannot
     // check its scope at this point
-    gjs_callback_trampoline_unref(trampoline);
     gjs_arg_unset<void*>(in_arg);
     return true;
 }
@@ -1006,6 +984,18 @@ static bool gjs_marshal_boxed_in_release(JSContext*, GjsArgumentCache* self,
     return true;
 }
 
+GJS_JSAPI_RETURN_CONVENTION
+static bool gjs_marshal_gvalue_in_maybe_release(JSContext* cx,
+                                                GjsArgumentCache* self,
+                                                GjsFunctionCallState* state,
+                                                GIArgument* in_arg,
+                                                GIArgument* out_arg) {
+    if (state->ignore_release.erase(in_arg))
+        return true;
+
+    return gjs_marshal_boxed_in_release(cx, self, state, in_arg, out_arg);
+}
+
 static void gjs_arg_cache_interface_free(GjsArgumentCache* self) {
     g_clear_pointer(&self->contents.object.info, g_base_info_unref);
 }
@@ -1098,10 +1088,10 @@ static const GjsArgumentMarshallers gvalue_in_marshallers = {
 };
 
 static const GjsArgumentMarshallers gvalue_in_transfer_none_marshallers = {
-    gjs_marshal_gvalue_in_in,  // in
-    gjs_marshal_skipped_out,  // out
-    gjs_marshal_boxed_in_release,  // release
-    gjs_arg_cache_interface_free,  // free
+    gjs_marshal_gvalue_in_in,             // in
+    gjs_marshal_skipped_out,              // out
+    gjs_marshal_gvalue_in_maybe_release,  // release
+    gjs_arg_cache_interface_free,         // free
 };
 
 static const GjsArgumentMarshallers gclosure_in_marshallers = {
@@ -1184,15 +1174,9 @@ static const GjsArgumentMarshallers boolean_in_marshallers = {
     gjs_marshal_skipped_release,  // release
 };
 
-static const GjsArgumentMarshallers integer_in_marshallers = {
-    gjs_marshal_integer_in_in,  // in
-    gjs_marshal_skipped_out,  // out
-    gjs_marshal_skipped_release,  // release
-};
-
-static const GjsArgumentMarshallers number_in_marshallers = {
-    gjs_marshal_number_in_in,  // in
-    gjs_marshal_skipped_out,  // out
+static const GjsArgumentMarshallers numeric_in_marshallers = {
+    gjs_marshal_numeric_in_in,    // in
+    gjs_marshal_skipped_out,      // out
     gjs_marshal_skipped_release,  // release
 };
 
@@ -1265,7 +1249,7 @@ static const GjsArgumentMarshallers caller_allocates_out_marshallers = {
 
 static inline void gjs_arg_cache_set_skip_all(GjsArgumentCache* self) {
     self->marshallers = &skip_all_marshallers;
-    self->skip_in = self->skip_out = true;
+    self->flags = (GjsArgumentFlags::SKIP_IN | GjsArgumentFlags::SKIP_OUT);
 }
 
 bool gjs_arg_cache_build_return(JSContext*, GjsArgumentCache* self,
@@ -1312,7 +1296,7 @@ bool gjs_arg_cache_build_return(JSContext*, GjsArgumentCache* self,
 
     // marshal_in is ignored for the return value, but skip_in is not (it is
     // used in the failure release path)
-    self->skip_in = true;
+    self->flags = (self->flags | GjsArgumentFlags::SKIP_IN);
     self->marshallers = &return_value_marshallers;
 
     return true;
@@ -1320,8 +1304,8 @@ bool gjs_arg_cache_build_return(JSContext*, GjsArgumentCache* self,
 
 static void gjs_arg_cache_build_enum_bounds(GjsArgumentCache* self,
                                             GIEnumInfo* enum_info) {
-    int64_t min = G_MAXINT64;
-    int64_t max = G_MININT64;
+    int64_t min = std::numeric_limits<int64_t>::max();
+    int64_t max = std::numeric_limits<int64_t>::min();
     int n = g_enum_info_get_n_values(enum_info);
     for (int i = 0; i < n; i++) {
         GjsAutoValueInfo value_info = g_enum_info_get_value(enum_info, i);
@@ -1340,7 +1324,11 @@ static void gjs_arg_cache_build_enum_bounds(GjsArgumentCache* self,
     // whether we have to compare them as signed.
     self->contents.enum_type.enum_min = static_cast<uint32_t>(min);
     self->contents.enum_type.enum_max = static_cast<uint32_t>(max);
-    self->is_unsigned = min >= 0 && max > G_MAXINT32;
+
+    if (min >= 0 && max > std::numeric_limits<int32_t>::max())
+        self->flags = (self->flags | GjsArgumentFlags::UNSIGNED);
+    else
+        self->flags = (self->flags & ~GjsArgumentFlags::UNSIGNED);
 }
 
 static void gjs_arg_cache_build_flags_mask(GjsArgumentCache* self,
@@ -1410,7 +1398,9 @@ static bool gjs_arg_cache_build_interface_in_arg(JSContext* cx,
             // marshallers know not to copy stuff if we don't need to.
 
             if (gtype == G_TYPE_VALUE) {
-                if (self->transfer == GI_TRANSFER_NOTHING && !is_instance_param)
+                if (self->arg_pos == GjsArgumentCache::INSTANCE_PARAM)
+                    self->marshallers = &boxed_in_marshallers;
+                else if (self->transfer == GI_TRANSFER_NOTHING && !is_instance_param)
                     self->marshallers = &gvalue_in_transfer_none_marshallers;
                 else
                     self->marshallers = &gvalue_in_marshallers;
@@ -1533,24 +1523,14 @@ static bool gjs_arg_cache_build_normal_in_arg(JSContext* cx,
         case GI_TYPE_TAG_INT8:
         case GI_TYPE_TAG_INT16:
         case GI_TYPE_TAG_INT32:
-            self->marshallers = &integer_in_marshallers;
-            self->contents.number.number_tag = tag;
-            self->is_unsigned = false;
-            break;
-
         case GI_TYPE_TAG_UINT8:
         case GI_TYPE_TAG_UINT16:
-            self->marshallers = &integer_in_marshallers;
-            self->contents.number.number_tag = tag;
-            self->is_unsigned = true;
-            break;
-
         case GI_TYPE_TAG_UINT32:
         case GI_TYPE_TAG_INT64:
         case GI_TYPE_TAG_UINT64:
         case GI_TYPE_TAG_FLOAT:
         case GI_TYPE_TAG_DOUBLE:
-            self->marshallers = &number_in_marshallers;
+            self->marshallers = &numeric_in_marshallers;
             self->contents.number.number_tag = tag;
             break;
 
@@ -1567,7 +1547,7 @@ static bool gjs_arg_cache_build_normal_in_arg(JSContext* cx,
                 self->marshallers = &string_in_transfer_none_marshallers;
             else
                 self->marshallers = &string_in_marshallers;
-            self->contents.string_is_filename = true;
+            self->flags = (self->flags | GjsArgumentFlags::FILENAME);
             break;
 
         case GI_TYPE_TAG_UTF8:
@@ -1575,7 +1555,7 @@ static bool gjs_arg_cache_build_normal_in_arg(JSContext* cx,
                 self->marshallers = &string_in_transfer_none_marshallers;
             else
                 self->marshallers = &string_in_marshallers;
-            self->contents.string_is_filename = false;
+            self->flags = (self->flags & ~GjsArgumentFlags::FILENAME);
             break;
 
         case GI_TYPE_TAG_INTERFACE: {
@@ -1641,16 +1621,24 @@ bool gjs_arg_cache_build_arg(JSContext* cx, GjsArgumentCache* self,
     self->arg_name = g_base_info_get_name(arg);
     g_arg_info_load_type(arg, &self->type_info);
     self->transfer = g_arg_info_get_ownership_transfer(arg);
-    self->nullable = g_arg_info_may_be_null(arg);
+
+    GjsArgumentFlags flags = GjsArgumentFlags::NONE;
+    if (g_arg_info_may_be_null(arg))
+        flags |= GjsArgumentFlags::MAY_BE_NULL;
+    if (g_arg_info_is_caller_allocates(arg))
+        flags |= GjsArgumentFlags::CALLER_ALLOCATES;
 
     if (direction == GI_DIRECTION_IN)
-        self->skip_out = true;
+        flags |= GjsArgumentFlags::SKIP_OUT;
     else if (direction == GI_DIRECTION_OUT)
-        self->skip_in = true;
+        flags |= GjsArgumentFlags::SKIP_IN;
     *inc_counter_out = true;
 
+    self->flags = flags;
+
     GITypeTag type_tag = g_type_info_get_tag(&self->type_info);
-    if (direction == GI_DIRECTION_OUT && g_arg_info_is_caller_allocates(arg)) {
+    if (direction == GI_DIRECTION_OUT &&
+        (flags & GjsArgumentFlags::CALLER_ALLOCATES)) {
         if (type_tag != GI_TYPE_TAG_INTERFACE) {
             gjs_throw(cx,
                       "Unsupported type %s for argument %s with (out "
@@ -1743,7 +1731,11 @@ bool gjs_arg_cache_build_arg(JSContext* cx, GjsArgumentCache* self,
         int length_pos = g_type_info_get_array_length(&self->type_info);
 
         if (length_pos >= 0) {
-            gjs_arg_cache_set_skip_all(&arguments[length_pos]);
+            GjsArgumentCache *cached_length = &arguments[length_pos];
+            bool skip_length =
+                !(cached_length->skip_in() && cached_length->skip_out());
+            if (skip_length)
+                gjs_arg_cache_set_skip_all(cached_length);
 
             if (direction == GI_DIRECTION_IN) {
                 self->marshallers = &c_array_in_marshallers;
@@ -1767,7 +1759,7 @@ bool gjs_arg_cache_build_arg(JSContext* cx, GjsArgumentCache* self,
             g_arg_info_load_type(&length_arg, &length_type);
             self->contents.array.length_tag = g_type_info_get_tag(&length_type);
 
-            if (length_pos < gi_index) {
+            if (length_pos < gi_index && skip_length) {
                 // we already collected length_pos, remove it
                 *inc_counter_out = false;
             }
