@@ -1,25 +1,6 @@
 /* -*- mode: C++; c-basic-offset: 4; indent-tabs-mode: nil; -*- */
-/*
- * Copyright (c) 2013 Giovanni Campagna <scampa.giovanni@gmail.com>
- *
- * Permission is hereby granted, free of charge, to any person obtaining a copy
- * of this software and associated documentation files (the "Software"), to
- * deal in the Software without restriction, including without limitation the
- * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
- * sell copies of the Software, and to permit persons to whom the Software is
- * furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice shall be included in
- * all copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
- * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
- * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
- * IN THE SOFTWARE.
- */
+// SPDX-License-Identifier: MIT OR LGPL-2.0-or-later
+// SPDX-FileCopyrightText: 2013 Giovanni Campagna <scampa.giovanni@gmail.com>
 
 #ifndef GI_ARG_CACHE_H_
 #define GI_ARG_CACHE_H_
@@ -36,6 +17,8 @@
 #include <js/RootingAPI.h>
 #include <js/TypeDecls.h>
 
+#include "gi/arg.h"
+#include "gjs/enum-utils.h"
 #include "gjs/macros.h"
 
 struct GjsFunctionCallState;
@@ -60,11 +43,8 @@ struct GjsArgumentCache {
     GITypeInfo type_info;
 
     uint8_t arg_pos;
-    bool skip_in : 1;
-    bool skip_out : 1;
     GITransfer transfer : 2;
-    bool nullable : 1;
-    bool is_unsigned : 1;  // number and enum only
+    GjsArgumentFlags flags : 5;
 
     union {
         // for explicit array only
@@ -99,9 +79,6 @@ struct GjsArgumentCache {
         } enum_type;
         unsigned flags_mask;
 
-        // string / filename
-        bool string_is_filename : 1;
-
         // out caller allocates (FIXME: should be in object)
         size_t caller_allocates_size;
     } contents;
@@ -145,16 +122,24 @@ struct GjsArgumentCache {
         arg_name = "instance parameter";
         // Some calls accept null for the instance, but generally in an object
         // oriented language it's wrong to call a method on null
-        nullable = false;
-        skip_out = true;
+        flags = GjsArgumentFlags::NONE | GjsArgumentFlags::SKIP_OUT;
     }
 
     void set_return_value() {
         arg_pos = RETURN_VALUE;
         arg_name = "return value";
-        nullable = false;  // We don't really care for return values
+        flags =
+            GjsArgumentFlags::NONE;  // We don't really care for return values
     }
     [[nodiscard]] bool is_return_value() { return arg_pos == RETURN_VALUE; }
+
+    constexpr bool skip_in() const {
+        return (flags & GjsArgumentFlags::SKIP_IN);
+    }
+
+    constexpr bool skip_out() const {
+        return (flags & GjsArgumentFlags::SKIP_OUT);
+    }
 };
 
 // This is a trick to print out the sizes of the structs at compile time, in
