@@ -1,34 +1,17 @@
 /* -*- mode: C++; c-basic-offset: 4; indent-tabs-mode: nil; -*- */
-/*
- * Copyright (c) 2008  litl, LLC
- *
- * Permission is hereby granted, free of charge, to any person obtaining a copy
- * of this software and associated documentation files (the "Software"), to
- * deal in the Software without restriction, including without limitation the
- * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
- * sell copies of the Software, and to permit persons to whom the Software is
- * furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice shall be included in
- * all copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
- * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
- * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
- * IN THE SOFTWARE.
- */
+// SPDX-License-Identifier: MIT OR LGPL-2.0-or-later
+// SPDX-FileCopyrightText: 2008 litl, LLC
 
 #include <config.h>
 
 #include <stdint.h>
 #include <stdlib.h>  // for exit
-#include <string.h>  // for strcmp, memset, size_t
+#include <string.h>  // for memset
 
-#include <new>
+#include <memory>  // for unique_ptr
 #include <string>
+#include <type_traits>
+#include <vector>
 
 #include <ffi.h>
 #include <girepository.h>
@@ -46,6 +29,7 @@
 #include <js/RootingAPI.h>
 #include <js/TypeDecls.h>
 #include <js/Value.h>
+#include <js/ValueArray.h>
 #include <js/Warnings.h>
 #include <jsapi.h>        // for HandleValueArray, JS_GetElement
 
@@ -85,30 +69,21 @@ extern struct JSClass gjs_function_class;
  * while it's in use, this list keeps track of ones that
  * will be freed the next time we invoke a C function.
  */
-static GSList *completed_trampolines = NULL;  /* GjsCallbackTrampoline */
+static std::vector<GjsAutoCallbackTrampoline> completed_trampolines;
 
 GJS_DEFINE_PRIV_FROM_JS(Function, gjs_function_class)
 
-void
-gjs_callback_trampoline_ref(GjsCallbackTrampoline *trampoline)
-{
-    trampoline->ref_count++;
+GjsCallbackTrampoline* gjs_callback_trampoline_ref(
+    GjsCallbackTrampoline* trampoline) {
+    g_atomic_ref_count_inc(&trampoline->ref_count);
+    return trampoline;
 }
 
 void
 gjs_callback_trampoline_unref(GjsCallbackTrampoline *trampoline)
 {
-    /* Not MT-safe, like all the rest of GJS */
-
-    trampoline->ref_count--;
-    if (trampoline->ref_count == 0) {
-        g_clear_pointer(&trampoline->js_function, g_closure_unref);
-        if (trampoline->info && trampoline->closure)
-            g_callable_info_free_closure(trampoline->info, trampoline->closure);
-        g_clear_pointer(&trampoline->info, g_base_info_unref);
-        g_free (trampoline->param_types);
-        g_slice_free(GjsCallbackTrampoline, trampoline);
-    }
+    if (g_atomic_ref_count_dec(&trampoline->ref_count))
+        delete trampoline;
 }
 
 template <typename T, GITypeTag TAG = GI_TYPE_TAG_VOID>
@@ -169,10 +144,9 @@ set_return_ffi_arg_from_giargument (GITypeInfo  *ret_type,
         break;
     case GI_TYPE_TAG_INTERFACE:
         {
-            GIBaseInfo* interface_info;
             GIInfoType interface_type;
 
-            interface_info = g_type_info_get_interface(ret_type);
+            GjsAutoBaseInfo interface_info(g_type_info_get_interface(ret_type));
             interface_type = g_base_info_get_type(interface_info);
 
             if (interface_type == GI_INFO_TYPE_ENUM ||
@@ -180,8 +154,6 @@ set_return_ffi_arg_from_giargument (GITypeInfo  *ret_type,
                 set_ffi_arg<int, GI_TYPE_TAG_INTERFACE>(result, return_value);
             else
                 set_ffi_arg<void*>(result, return_value);
-
-            g_base_info_unref(interface_info);
         }
         break;
     case GI_TYPE_TAG_UINT64:
@@ -212,20 +184,14 @@ set_return_ffi_arg_from_giargument (GITypeInfo  *ret_type,
     }
 }
 
-static void
-warn_about_illegal_js_callback(const GjsCallbackTrampoline *trampoline,
-                               const char *when,
-                               const char *reason)
-{
+void GjsCallbackTrampoline::warn_about_illegal_js_callback(const char* when,
+                                                           const char* reason) {
     g_critical("Attempting to run a JS callback %s. This is most likely caused "
                "by %s. Because it would crash the application, it has been "
                "blocked.", when, reason);
-    if (trampoline->info) {
-        const char *name = g_base_info_get_name(trampoline->info);
-        g_critical("The offending callback was %s()%s.", name,
-                   trampoline->is_vfunc ? ", a vfunc" : "");
-    }
-    return;
+    if (m_info)
+        g_critical("The offending callback was %s()%s.", m_info.name(),
+                   m_is_vfunc ? ", a vfunc" : "");
 }
 
 /* This is our main entry point for ffi_closure callbacks.
@@ -235,56 +201,63 @@ warn_about_illegal_js_callback(const GjsCallbackTrampoline *trampoline,
  * In other words, everything we need to call the JS function and
  * getting the return value back.
  */
-static void gjs_callback_closure(ffi_cif* cif [[maybe_unused]], void* result,
-                                 void** ffi_args, void* data) {
+void GjsCallbackTrampoline::callback_closure(GIArgument** args, void* result) {
     JSContext *context;
-    GjsCallbackTrampoline *trampoline;
-    int i, n_args, n_jsargs, n_outargs, c_args_offset = 0;
     GITypeInfo ret_type;
-    bool success = false;
-    auto args = reinterpret_cast<GIArgument **>(ffi_args);
 
-    trampoline = (GjsCallbackTrampoline *) data;
-    g_assert(trampoline);
-    gjs_callback_trampoline_ref(trampoline);
-
-    if (G_UNLIKELY(!gjs_closure_is_valid(trampoline->js_function))) {
-        warn_about_illegal_js_callback(trampoline, "during shutdown",
+    if (G_UNLIKELY(!gjs_closure_is_valid(m_js_function))) {
+        warn_about_illegal_js_callback(
+            "during shutdown",
             "destroying a Clutter actor or GTK widget with ::destroy signal "
             "connected, or using the destroy(), dispose(), or remove() vfuncs");
         gjs_dumpstack();
-        gjs_callback_trampoline_unref(trampoline);
         return;
     }
 
-    context = gjs_closure_get_context(trampoline->js_function);
+    context = gjs_closure_get_context(m_js_function);
     GjsContextPrivate* gjs = GjsContextPrivate::from_cx(context);
     if (G_UNLIKELY(gjs->sweeping())) {
-        warn_about_illegal_js_callback(trampoline, "during garbage collection",
+        warn_about_illegal_js_callback(
+            "during garbage collection",
             "destroying a Clutter actor or GTK widget with ::destroy signal "
             "connected, or using the destroy(), dispose(), or remove() vfuncs");
         gjs_dumpstack();
-        gjs_callback_trampoline_unref(trampoline);
         return;
     }
 
     if (G_UNLIKELY(!gjs->is_owner_thread())) {
-        warn_about_illegal_js_callback(trampoline, "on a different thread",
-            "an API not intended to be used in JS");
-        gjs_callback_trampoline_unref(trampoline);
+        warn_about_illegal_js_callback("on a different thread",
+                                       "an API not intended to be used in JS");
         return;
     }
 
-    JSAutoRealm ar(context, JS_GetFunctionObject(gjs_closure_get_callable(
-                                trampoline->js_function)));
-
-    bool can_throw_gerror = g_callable_info_can_throw_gerror(trampoline->info);
-    n_args = g_callable_info_get_n_args(trampoline->info);
+    JSAutoRealm ar(
+        context, JS_GetFunctionObject(gjs_closure_get_callable(m_js_function)));
 
+    int n_args = m_param_types.size();
     g_assert(n_args >= 0);
 
+    struct AutoCallbackData {
+        AutoCallbackData(GjsCallbackTrampoline* trampoline,
+                         GjsContextPrivate* gjs)
+            : trampoline(trampoline), gjs(gjs) {}
+        ~AutoCallbackData() {
+            if (trampoline->m_scope == GI_SCOPE_TYPE_ASYNC) {
+                // We don't release the trampoline here as we've an extra ref
+                // that has been set in gjs_marshal_callback_in()
+                completed_trampolines.emplace_back(trampoline);
+            }
+            gjs->schedule_gc_if_needed();
+        }
+
+        GjsCallbackTrampoline* trampoline;
+        GjsContextPrivate* gjs;
+    };
+
+    AutoCallbackData callback_data(this, gjs);
     JS::RootedObject this_object(context);
-    if (trampoline->is_vfunc) {
+    int c_args_offset = 0;
+    if (m_is_vfunc) {
         GObject* gobj = G_OBJECT(gjs_arg_get<GObject*>(args[0]));
         if (gobj) {
             this_object = ObjectInstance::wrapper_from_gobject(context, gobj);
@@ -299,23 +272,87 @@ static void gjs_callback_closure(ffi_cif* cif [[maybe_unused]], void* result,
         c_args_offset = 1;
     }
 
-    n_outargs = 0;
+    JS::RootedValue rval(context);
+
+    g_callable_info_load_return_type(m_info, &ret_type);
+    GIArgument* error_argument = nullptr;
+
+    if (g_callable_info_can_throw_gerror(m_info))
+        error_argument = args[n_args + c_args_offset];
+
+    if (!callback_closure_inner(context, this_object, &rval, args, &ret_type,
+                                n_args, c_args_offset, result)) {
+        if (!JS_IsExceptionPending(context)) {
+            // "Uncatchable" exception thrown, we have to exit. We may be in a
+            // main loop, or maybe not, but there's no way to tell, so we have
+            // to exit here instead of propagating the exception back to the
+            // original calling JS code.
+            uint8_t code;
+            if (gjs->should_exit(&code))
+                exit(code);
+
+            // Some other uncatchable exception, e.g. out of memory
+            JSFunction* fn = gjs_closure_get_callable(m_js_function);
+            g_error("Function %s (%s.%s) terminated with uncatchable exception",
+                    gjs_debug_string(JS_GetFunctionDisplayId(fn)).c_str(),
+                    m_info.ns(), m_info.name());
+        }
+
+        // Fill in the result with some hopefully neutral value
+        if (g_type_info_get_tag(&ret_type) != GI_TYPE_TAG_VOID) {
+            GIArgument argument = {};
+            g_callable_info_load_return_type(m_info, &ret_type);
+            gjs_gi_argument_init_default(&ret_type, &argument);
+            set_return_ffi_arg_from_giargument(&ret_type, result, &argument);
+        }
+
+        // If the callback has a GError** argument and invoking the closure
+        // returned an error, try to make a GError from it
+        if (error_argument && rval.isObject()) {
+            JS::RootedObject exc_object(context, &rval.toObject());
+            GError* local_error =
+                gjs_gerror_make_from_error(context, exc_object);
+
+            if (local_error) {
+                // the GError ** pointer is the last argument, and is not
+                // included in the n_args
+                auto* gerror = gjs_arg_get<GError**>(error_argument);
+                g_propagate_error(gerror, local_error);
+                JS_ClearPendingException(context);  // don't log
+            }
+        } else if (!rval.isUndefined()) {
+            JS_SetPendingException(context, rval);
+        }
+        gjs_log_exception_uncaught(context);
+    }
+}
+
+inline GIArgument* get_argument_for_arg_info(GIArgInfo* arg_info,
+                                             GIArgument** args, int index) {
+    if (!g_arg_info_is_caller_allocates(arg_info))
+        return *reinterpret_cast<GIArgument**>(args[index]);
+    else
+        return args[index];
+}
+
+bool GjsCallbackTrampoline::callback_closure_inner(
+    JSContext* context, JS::HandleObject this_object,
+    JS::MutableHandleValue rval, GIArgument** args, GITypeInfo* ret_type,
+    int n_args, int c_args_offset, void* result) {
+    int n_outargs = 0;
     JS::RootedValueVector jsargs(context);
 
     if (!jsargs.reserve(n_args))
         g_error("Unable to reserve space for vector");
 
-    JS::RootedValue rval(context);
+    bool ret_type_is_void = g_type_info_get_tag(ret_type) == GI_TYPE_TAG_VOID;
 
-    g_callable_info_load_return_type(trampoline->info, &ret_type);
-    bool ret_type_is_void = g_type_info_get_tag (&ret_type) == GI_TYPE_TAG_VOID;
-
-    for (i = 0, n_jsargs = 0; i < n_args; i++) {
+    for (int i = 0, n_jsargs = 0; i < n_args; i++) {
         GIArgInfo arg_info;
         GITypeInfo type_info;
         GjsParamType param_type;
 
-        g_callable_info_load_arg(trampoline->info, i, &arg_info);
+        g_callable_info_load_arg(m_info, i, &arg_info);
         g_arg_info_load_type(&arg_info, &type_info);
 
         /* Skip void * arguments */
@@ -330,7 +367,7 @@ static void gjs_callback_closure(ffi_cif* cif [[maybe_unused]], void* result,
         if (g_arg_info_get_direction(&arg_info) == GI_DIRECTION_INOUT)
             n_outargs++;
 
-        param_type = trampoline->param_types[i];
+        param_type = m_param_types[i];
 
         switch (param_type) {
             case PARAM_SKIPPED:
@@ -341,12 +378,13 @@ static void gjs_callback_closure(ffi_cif* cif [[maybe_unused]], void* result,
                 GITypeInfo arg_type_info;
                 JS::RootedValue length(context);
 
-                g_callable_info_load_arg(trampoline->info, array_length_pos, &array_length_arg);
+                g_callable_info_load_arg(m_info, array_length_pos,
+                                         &array_length_arg);
                 g_arg_info_load_type(&array_length_arg, &arg_type_info);
                 if (!gjs_value_from_g_argument(context, &length, &arg_type_info,
                                                args[array_length_pos + c_args_offset],
                                                true))
-                    goto out;
+                    return false;
 
                 if (!jsargs.growBy(1))
                     g_error("Unable to grow vector");
@@ -355,7 +393,7 @@ static void gjs_callback_closure(ffi_cif* cif [[maybe_unused]], void* result,
                                                    &type_info,
                                                    args[i + c_args_offset],
                                                    length.toInt32()))
-                    goto out;
+                    return false;
                 break;
             }
             case PARAM_NORMAL: {
@@ -363,12 +401,13 @@ static void gjs_callback_closure(ffi_cif* cif [[maybe_unused]], void* result,
                     g_error("Unable to grow vector");
 
                 GIArgument* arg = args[i + c_args_offset];
-                if (g_arg_info_get_direction(&arg_info) == GI_DIRECTION_INOUT)
+                if (g_arg_info_get_direction(&arg_info) == GI_DIRECTION_INOUT &&
+                    !g_arg_info_is_caller_allocates(&arg_info))
                     arg = *reinterpret_cast<GIArgument**>(arg);
 
                 if (!gjs_value_from_g_argument(context, jsargs[n_jsargs++],
                                                &type_info, arg, false))
-                    goto out;
+                    return false;
                 break;
             }
             case PARAM_CALLBACK:
@@ -381,9 +420,8 @@ static void gjs_callback_closure(ffi_cif* cif [[maybe_unused]], void* result,
         }
     }
 
-    if (!gjs_closure_invoke(trampoline->js_function, this_object, jsargs, &rval,
-                            true))
-        goto out;
+    if (!gjs_closure_invoke(m_js_function, this_object, jsargs, rval, true))
+        return false;
 
     if (n_outargs == 0 && ret_type_is_void) {
         /* void return value, no out args, nothing to do */
@@ -391,51 +429,44 @@ static void gjs_callback_closure(ffi_cif* cif [[maybe_unused]], void* result,
         GIArgument argument;
         GITransfer transfer;
 
-        transfer = g_callable_info_get_caller_owns (trampoline->info);
+        transfer = g_callable_info_get_caller_owns(m_info);
         /* non-void return value, no out args. Should
          * be a single return value. */
-        if (!gjs_value_to_g_argument(context,
-                                     rval,
-                                     &ret_type,
-                                     "callback",
-                                     GJS_ARGUMENT_RETURN_VALUE,
-                                     transfer,
-                                     true,
-                                     &argument))
-            goto out;
+        if (!gjs_value_to_g_argument(context, rval, ret_type, "callback",
+                                     GJS_ARGUMENT_RETURN_VALUE, transfer,
+                                     GjsArgumentFlags::MAY_BE_NULL, &argument))
+            return false;
 
-        set_return_ffi_arg_from_giargument(&ret_type,
-                                           result,
-                                           &argument);
+        set_return_ffi_arg_from_giargument(ret_type, result, &argument);
     } else if (n_outargs == 1 && ret_type_is_void) {
         /* void return value, one out args. Should
          * be a single return value. */
-        for (i = 0; i < n_args; i++) {
+        for (int i = 0; i < n_args; i++) {
             GIArgInfo arg_info;
-            g_callable_info_load_arg(trampoline->info, i, &arg_info);
+            g_callable_info_load_arg(m_info, i, &arg_info);
             if (g_arg_info_get_direction(&arg_info) == GI_DIRECTION_IN)
                 continue;
 
             if (!gjs_value_to_arg(context, rval, &arg_info,
-                                  *reinterpret_cast<GIArgument **>(args[i + c_args_offset])))
-                goto out;
+                                  get_argument_for_arg_info(&arg_info, args,
+                                                            i + c_args_offset)))
+                return false;
 
             break;
         }
     } else {
         bool is_array = rval.isObject();
         if (!JS::IsArrayObject(context, rval, &is_array))
-            goto out;
+            return false;
 
         if (!is_array) {
-            JSFunction* fn = gjs_closure_get_callable(trampoline->js_function);
+            JSFunction* fn = gjs_closure_get_callable(m_js_function);
             gjs_throw(context,
                       "Function %s (%s.%s) returned unexpected value, "
                       "expecting an Array",
                       gjs_debug_string(JS_GetFunctionDisplayId(fn)).c_str(),
-                      g_base_info_get_namespace(trampoline->info),
-                      g_base_info_get_name(trampoline->info));
-            goto out;
+                      m_info.ns(), m_info.name());
+            return false;
         }
 
         JS::RootedValue elem(context);
@@ -446,126 +477,92 @@ static void gjs_callback_closure(ffi_cif* cif [[maybe_unused]], void* result,
 
         if (!ret_type_is_void) {
             GIArgument argument;
-            GITransfer transfer = g_callable_info_get_caller_owns(trampoline->info);
+            GITransfer transfer = g_callable_info_get_caller_owns(m_info);
 
             if (!JS_GetElement(context, out_array, elem_idx, &elem))
-                goto out;
+                return false;
 
-            if (!gjs_value_to_g_argument(context, elem, &ret_type, "callback",
+            if (!gjs_value_to_g_argument(context, elem, ret_type, "callback",
                                          GJS_ARGUMENT_RETURN_VALUE, transfer,
-                                         true, &argument))
-                goto out;
+                                         GjsArgumentFlags::MAY_BE_NULL,
+                                         &argument))
+                return false;
 
-            set_return_ffi_arg_from_giargument(&ret_type,
-                                               result,
-                                               &argument);
+            set_return_ffi_arg_from_giargument(ret_type, result, &argument);
 
             elem_idx++;
         }
 
-        for (i = 0; i < n_args; i++) {
+        for (int i = 0; i < n_args; i++) {
             GIArgInfo arg_info;
-            g_callable_info_load_arg(trampoline->info, i, &arg_info);
+            g_callable_info_load_arg(m_info, i, &arg_info);
             if (g_arg_info_get_direction(&arg_info) == GI_DIRECTION_IN)
                 continue;
 
             if (!JS_GetElement(context, out_array, elem_idx, &elem))
-                goto out;
+                return false;
 
             if (!gjs_value_to_arg(context, elem, &arg_info,
-                                  *(GIArgument **)args[i + c_args_offset]))
-                goto out;
+                                  get_argument_for_arg_info(&arg_info, args,
+                                                            i + c_args_offset)))
+                return false;
 
             elem_idx++;
         }
     }
 
-    success = true;
-
-out:
-    if (!success) {
-        if (!JS_IsExceptionPending(context)) {
-            /* "Uncatchable" exception thrown, we have to exit. We may be in a
-             * main loop, or maybe not, but there's no way to tell, so we have
-             * to exit here instead of propagating the exception back to the
-             * original calling JS code. */
-            uint8_t code;
-            if (gjs->should_exit(&code))
-                exit(code);
-
-            /* Some other uncatchable exception, e.g. out of memory */
-            JSFunction* fn = gjs_closure_get_callable(trampoline->js_function);
-            g_error("Function %s (%s.%s) terminated with uncatchable exception",
-                    gjs_debug_string(JS_GetFunctionDisplayId(fn)).c_str(),
-                    g_base_info_get_namespace(trampoline->info),
-                    g_base_info_get_name(trampoline->info));
-        }
-
-        /* Fill in the result with some hopefully neutral value */
-        if (!ret_type_is_void) {
-            GIArgument argument = {};
-            g_callable_info_load_return_type(trampoline->info, &ret_type);
-            gjs_gi_argument_init_default(&ret_type, &argument);
-            set_return_ffi_arg_from_giargument(&ret_type, result, &argument);
-        }
-
-        /* If the callback has a GError** argument and invoking the closure
-         * returned an error, try to make a GError from it */
-        if (can_throw_gerror && rval.isObject()) {
-            JS::RootedObject exc_object(context, &rval.toObject());
-            GError *local_error = gjs_gerror_make_from_error(context, exc_object);
-
-            if (local_error) {
-                /* the GError ** pointer is the last argument, and is not
-                 * included in the n_args */
-                GIArgument *error_argument = args[n_args + c_args_offset];
-                auto* gerror = gjs_arg_get<GError**>(error_argument);
-                g_propagate_error(gerror, local_error);
-                JS_ClearPendingException(context);  /* don't log */
-            }
-        } else if (!rval.isUndefined()) {
-            JS_SetPendingException(context, rval);
-        }
-        gjs_log_exception_uncaught(context);
-    }
-
-    if (trampoline->scope == GI_SCOPE_TYPE_ASYNC) {
-        completed_trampolines = g_slist_prepend(completed_trampolines, trampoline);
-    }
-
-    gjs_callback_trampoline_unref(trampoline);
-    gjs->schedule_gc_if_needed();
+    return true;
 }
 
 GjsCallbackTrampoline* gjs_callback_trampoline_new(
     JSContext* context, JS::HandleFunction function,
     GICallableInfo* callable_info, GIScopeType scope, bool has_scope_object,
     bool is_vfunc) {
-    GjsCallbackTrampoline *trampoline;
-    int n_args, i;
-
     g_assert(function);
 
-    trampoline = g_slice_new(GjsCallbackTrampoline);
-    new (trampoline) GjsCallbackTrampoline();
-    trampoline->ref_count = 1;
-    trampoline->info = callable_info;
-    g_base_info_ref((GIBaseInfo*)trampoline->info);
+    GjsAutoCallbackTrampoline trampoline =
+        new GjsCallbackTrampoline(callable_info, scope, is_vfunc);
 
-    /* Analyze param types and directions, similarly to init_cached_function_data */
-    n_args = g_callable_info_get_n_args(trampoline->info);
-    trampoline->param_types = g_new0(GjsParamType, n_args);
+    if (!trampoline->initialize(context, function, has_scope_object))
+        return nullptr;
 
-    for (i = 0; i < n_args; i++) {
+    return trampoline.release();
+}
+
+GjsCallbackTrampoline::GjsCallbackTrampoline(GICallableInfo* callable_info,
+                                             GIScopeType scope, bool is_vfunc)
+    : m_info(callable_info, GjsAutoTakeOwnership()),
+      m_scope(scope),
+      m_param_types(g_callable_info_get_n_args(callable_info), {}),
+      m_is_vfunc(is_vfunc) {
+    g_atomic_ref_count_init(&ref_count);
+}
+
+GjsCallbackTrampoline::~GjsCallbackTrampoline() {
+    g_assert(g_atomic_ref_count_compare(&ref_count, 0));
+
+    if (m_info && m_closure)
+        g_callable_info_free_closure(m_info, m_closure);
+}
+
+bool GjsCallbackTrampoline::initialize(JSContext* cx,
+                                       JS::HandleFunction function,
+                                       bool has_scope_object) {
+    g_assert(!m_js_function);
+    g_assert(!m_closure);
+
+    /* Analyze param types and directions, similarly to
+     * init_cached_function_data */
+    for (size_t i = 0; i < m_param_types.size(); i++) {
         GIDirection direction;
         GIArgInfo arg_info;
         GITypeInfo type_info;
         GITypeTag type_tag;
 
-        if (trampoline->param_types[i] == PARAM_SKIPPED)
+        if (m_param_types[i] == PARAM_SKIPPED)
             continue;
 
-        g_callable_info_load_arg(trampoline->info, i, &arg_info);
+        g_callable_info_load_arg(m_info, i, &arg_info);
         g_arg_info_load_type(&arg_info, &type_info);
 
         direction = g_arg_info_get_direction(&arg_info);
@@ -577,62 +574,68 @@ GjsCallbackTrampoline* gjs_callback_trampoline_new(
         }
 
         if (type_tag == GI_TYPE_TAG_INTERFACE) {
-            GIBaseInfo* interface_info;
             GIInfoType interface_type;
 
-            interface_info = g_type_info_get_interface(&type_info);
+            GjsAutoBaseInfo interface_info =
+                g_type_info_get_interface(&type_info);
             interface_type = g_base_info_get_type(interface_info);
             if (interface_type == GI_INFO_TYPE_CALLBACK) {
-                gjs_throw(context,
+                gjs_throw(cx,
                           "%s %s accepts another callback as a parameter. This "
                           "is not supported",
-                          is_vfunc ? "VFunc" : "Callback",
-                          g_base_info_get_name(callable_info));
-                g_base_info_unref(interface_info);
-                gjs_callback_trampoline_unref(trampoline);
-                return NULL;
+                          m_is_vfunc ? "VFunc" : "Callback", m_info.name());
+                return false;
             }
-            g_base_info_unref(interface_info);
         } else if (type_tag == GI_TYPE_TAG_ARRAY) {
             if (g_type_info_get_array_type(&type_info) == GI_ARRAY_TYPE_C) {
                 int array_length_pos = g_type_info_get_array_length(&type_info);
 
-                if (array_length_pos >= 0 && array_length_pos < n_args) {
+                if (array_length_pos < 0)
+                    continue;
+
+                if (static_cast<size_t>(array_length_pos) <
+                    m_param_types.size()) {
                     GIArgInfo length_arg_info;
 
-                    g_callable_info_load_arg(trampoline->info, array_length_pos, &length_arg_info);
+                    g_callable_info_load_arg(m_info, array_length_pos,
+                                             &length_arg_info);
                     if (g_arg_info_get_direction(&length_arg_info) != direction) {
-                        gjs_throw(context,
+                        gjs_throw(cx,
                                   "%s %s has an array with different-direction "
                                   "length argument. This is not supported",
-                                  is_vfunc ? "VFunc" : "Callback",
-                                  g_base_info_get_name(callable_info));
-                        gjs_callback_trampoline_unref(trampoline);
-                        return NULL;
+                                  m_is_vfunc ? "VFunc" : "Callback",
+                                  m_info.name());
+                        return false;
                     }
 
-                    trampoline->param_types[array_length_pos] = PARAM_SKIPPED;
-                    trampoline->param_types[i] = PARAM_ARRAY;
+                    m_param_types[array_length_pos] = PARAM_SKIPPED;
+                    m_param_types[i] = PARAM_ARRAY;
                 }
             }
         }
     }
 
-    trampoline->closure = g_callable_info_prepare_closure(callable_info, &trampoline->cif,
-                                                          gjs_callback_closure, trampoline);
+    m_closure = g_callable_info_prepare_closure(
+        m_info, &m_cif,
+        [](ffi_cif*, void* result, void** ffi_args, void* data) {
+            auto** args = reinterpret_cast<GIArgument**>(ffi_args);
+            g_assert(data && "Trampoline data is not set");
+            GjsAutoCallbackTrampoline trampoline(
+                static_cast<GjsCallbackTrampoline*>(data),
+                GjsAutoTakeOwnership());
+
+            trampoline->callback_closure(args, result);
+        },
+        this);
 
     // The rule is:
     // - notify callbacks in GObject methods are traced from the scope object
     // - async and call callbacks, and other notify callbacks, are rooted
     // - vfuncs are traced from the GObject prototype
-    bool should_root = scope != GI_SCOPE_TYPE_NOTIFIED || !has_scope_object;
-    trampoline->js_function = gjs_closure_new(
-        context, function, g_base_info_get_name(callable_info), should_root);
+    bool should_root = m_scope != GI_SCOPE_TYPE_NOTIFIED || !has_scope_object;
+    m_js_function = gjs_closure_new(cx, function, m_info.name(), should_root);
 
-    trampoline->scope = scope;
-    trampoline->is_vfunc = is_vfunc;
-
-    return trampoline;
+    return true;
 }
 
 /* Intended for error messages. Return value must be freed */
@@ -647,23 +650,12 @@ GjsCallbackTrampoline* gjs_callback_trampoline_new(
                            g_base_info_get_name(function->info));
 }
 
-static void
-complete_async_calls(void)
-{
-    if (completed_trampolines) {
-        for (GSList *iter = completed_trampolines; iter; iter = iter->next) {
-            auto trampoline = static_cast<GjsCallbackTrampoline *>(iter->data);
-            gjs_callback_trampoline_unref(trampoline);
-        }
-        g_slist_free(completed_trampolines);
-        completed_trampolines = nullptr;
-    }
-}
+void gjs_function_clear_async_closures() { completed_trampolines.clear(); }
 
 static void* get_return_ffi_pointer_from_giargument(
     GjsArgumentCache* return_arg, GIFFIReturnValue* return_value) {
     // This should be the inverse of gi_type_info_extract_ffi_return_value().
-    if (return_arg->skip_out)
+    if (return_arg->skip_out())
         return nullptr;
 
     // FIXME: Note that v_long and v_ulong don't have type-safe template
@@ -729,12 +721,6 @@ static bool gjs_invoke_c_function(JSContext* context, Function* function,
     bool is_method;
     JS::RootedValueVector return_values(context);
 
-    /* Because we can't free a closure while we're in it, we defer
-     * freeing until the next time a C function is invoked.  What
-     * we should really do instead is queue it for a GC thread.
-     */
-    complete_async_calls();
-
     is_method = g_callable_info_is_method(function->info);
     can_throw_gerror = g_callable_info_can_throw_gerror(function->info);
 
@@ -790,18 +776,9 @@ static bool gjs_invoke_c_function(JSContext* context, Function* function,
     //
     // Use gi_arg_pos to index inside the GIArgument array. Use ffi_arg_pos to
     // index inside ffi_arg_pointers.
-    GjsFunctionCallState state(context);
-    if (is_method) {
-        state.in_cvalues = g_newa(GIArgument, gi_argc + 2) + 2;
-        state.out_cvalues = g_newa(GIArgument, gi_argc + 2) + 2;
-        state.inout_original_cvalues = g_newa(GIArgument, gi_argc + 2) + 2;
-    } else {
-        state.in_cvalues = g_newa(GIArgument, gi_argc + 1) + 1;
-        state.out_cvalues = g_newa(GIArgument, gi_argc + 1) + 1;
-        state.inout_original_cvalues = g_newa(GIArgument, gi_argc + 1) + 1;
-    }
+    GjsFunctionCallState state(context, function->info, gi_argc);
 
-    void** ffi_arg_pointers = g_newa(void*, ffi_argc);
+    auto ffi_arg_pointers = std::make_unique<void*[]>(ffi_argc);
 
     failed = false;
     unsigned ffi_arg_pos = 0;  // index into ffi_arg_pointers
@@ -865,7 +842,7 @@ static bool gjs_invoke_c_function(JSContext* context, Function* function,
             break;
         }
 
-        if (!cache->skip_in)
+        if (!cache->skip_in())
             js_arg_pos++;
 
         processed_c_args++;
@@ -896,7 +873,7 @@ static bool gjs_invoke_c_function(JSContext* context, Function* function,
     return_value_p = get_return_ffi_pointer_from_giargument(
         &function->arguments[-1], &return_value);
     ffi_call(&(function->invoker.cif), FFI_FN(function->invoker.native_address),
-             return_value_p, ffi_arg_pointers);
+             return_value_p, ffi_arg_pointers.get());
 
     /* Return value and out arguments are valid only if invocation doesn't
      * return error. In arguments need to be released always.
@@ -910,7 +887,7 @@ static bool gjs_invoke_c_function(JSContext* context, Function* function,
     if (!r_value)
         args.rval().setUndefined();
 
-    if (!function->arguments[-1].skip_out) {
+    if (!function->arguments[-1].skip_out()) {
         gi_type_info_extract_ffi_return_value(
             &function->arguments[-1].type_info, &return_value,
             &state.out_cvalues[-1]);
@@ -936,7 +913,7 @@ static bool gjs_invoke_c_function(JSContext* context, Function* function,
             }
         }
 
-        if (!cache->skip_out) {
+        if (!cache->skip_out()) {
             if (!r_value) {
                 if (!return_values.append(js_out_arg)) {
                     JS_ReportOutOfMemory(context);
@@ -980,7 +957,7 @@ release:
             processed_c_args);
 
         // Only process in or inout arguments if we failed, the rest is garbage
-        if (failed && cache->skip_in)
+        if (failed && cache->skip_in())
             continue;
 
         // Save the return GIArgument if it was requested
@@ -1097,7 +1074,7 @@ static void function_finalize(JSFreeOp*, JSObject* obj) {
     uninit_cached_function_data(priv);
 
     GJS_DEC_COUNTER(function);
-    g_slice_free(Function, priv);
+    g_free(priv);
 }
 
 GJS_JSAPI_RETURN_CONVENTION
@@ -1119,8 +1096,6 @@ function_to_string (JSContext *context,
 {
     GJS_GET_PRIV(context, argc, vp, rec, to, Function, priv);
     int i, n_args, n_jsargs;
-    GString *arg_names_str;
-    gchar *arg_names;
 
     if (priv == NULL) {
         JSString* retval = JS_NewStringCopyZ(context, "function () {\n}");
@@ -1132,33 +1107,30 @@ function_to_string (JSContext *context,
 
     n_args = g_callable_info_get_n_args(priv->info);
     n_jsargs = 0;
-    arg_names_str = g_string_new("");
+    std::string arg_names;
     for (i = 0; i < n_args; i++) {
-        if (priv->arguments[i].skip_in)
+        if (priv->arguments[i].skip_in())
             continue;
 
         if (n_jsargs > 0)
-            g_string_append(arg_names_str, ", ");
+            arg_names += ", ";
 
         n_jsargs++;
-        g_string_append(arg_names_str, priv->arguments[i].arg_name);
+        arg_names += priv->arguments[i].arg_name;
     }
-    arg_names = g_string_free(arg_names_str, false);
 
     GjsAutoChar descr;
     if (g_base_info_get_type(priv->info) == GI_INFO_TYPE_FUNCTION) {
         descr = g_strdup_printf(
             "function %s(%s) {\n\t/* wrapper for native symbol %s(); */\n}",
-            g_base_info_get_name(priv->info), arg_names,
+            g_base_info_get_name(priv->info), arg_names.c_str(),
             g_function_info_get_symbol(priv->info));
     } else {
         descr = g_strdup_printf(
             "function %s(%s) {\n\t/* wrapper for native symbol */\n}",
-            g_base_info_get_name(priv->info), arg_names);
+            g_base_info_get_name(priv->info), arg_names.c_str());
     }
 
-    g_free(arg_names);
-
     return gjs_string_from_utf8(context, descr, rec.rval());
 }
 
@@ -1266,7 +1238,7 @@ init_cached_function_data (JSContext      *context,
         GIDirection direction;
         GIArgInfo arg_info;
 
-        if (arguments[i].skip_in || arguments[i].skip_out)
+        if (arguments[i].skip_in() || arguments[i].skip_out())
             continue;
 
         g_callable_info_load_arg((GICallableInfo*) info, i, &arg_info);
@@ -1322,7 +1294,7 @@ function_new(JSContext      *context,
         return NULL;
     }
 
-    priv = g_slice_new0(Function);
+    priv = g_new0(Function, 1);
 
     GJS_INC_COUNTER(function);
 
