@@ -1,26 +1,7 @@
 /* -*- mode: C++; c-basic-offset: 4; indent-tabs-mode: nil; -*- */
-/*
- * Copyright (c) 2013       Intel Corporation
- * Copyright (c) 2008-2010  litl, LLC
- *
- * Permission is hereby granted, free of charge, to any person obtaining a copy
- * of this software and associated documentation files (the "Software"), to
- * deal in the Software without restriction, including without limitation the
- * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
- * sell copies of the Software, and to permit persons to whom the Software is
- * furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice shall be included in
- * all copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
- * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
- * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
- * IN THE SOFTWARE.
- */
+// SPDX-License-Identifier: MIT OR LGPL-2.0-or-later
+// SPDX-FileCopyrightText: 2013 Intel Corporation
+// SPDX-FileCopyrightText: 2008-2010 litl, LLC
 
 #ifndef GI_FUNDAMENTAL_H_
 #define GI_FUNDAMENTAL_H_
@@ -33,6 +14,7 @@
 #include <js/TypeDecls.h>
 
 #include "gi/wrapperutils.h"
+#include "gjs/jsapi-util.h"  // for GjsAutoCallableInfo
 #include "gjs/macros.h"
 #include "util/log.h"
 
@@ -55,18 +37,13 @@ class FundamentalBase
  protected:
     explicit FundamentalBase(FundamentalPrototype* proto = nullptr)
         : GIWrapperBase(proto) {}
-    ~FundamentalBase(void) {}
 
-    static const GjsDebugTopic debug_topic = GJS_DEBUG_GFUNDAMENTAL;
-    static constexpr const char* debug_tag = "fundamental";
+    static constexpr GjsDebugTopic DEBUG_TOPIC = GJS_DEBUG_GFUNDAMENTAL;
+    static constexpr const char* DEBUG_TAG = "fundamental";
 
     static const struct JSClassOps class_ops;
     static const struct JSClass klass;
 
-    // Helper methods
-
-    [[nodiscard]] const char* to_string_kind() const { return "fundamental"; }
-
     // Public API
 
  public:
@@ -86,7 +63,7 @@ class FundamentalPrototype
     GIObjectInfoUnrefFunction m_unref_function;
     GIObjectInfoGetValueFunction m_get_value_function;
     GIObjectInfoSetValueFunction m_set_value_function;
-    GICallableInfo* m_constructor_info;
+    GjsAutoCallableInfo m_constructor_info;
 
     explicit FundamentalPrototype(GIObjectInfo* info, GType gtype);
     ~FundamentalPrototype(void);
