@@ -1,25 +1,6 @@
 /* -*- mode: C++; c-basic-offset: 4; indent-tabs-mode: nil; -*- */
-/*
- * Copyright (c) 2008  litl, LLC
- *
- * Permission is hereby granted, free of charge, to any person obtaining a copy
- * of this software and associated documentation files (the "Software"), to
- * deal in the Software without restriction, including without limitation the
- * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
- * sell copies of the Software, and to permit persons to whom the Software is
- * furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice shall be included in
- * all copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
- * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
- * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
- * IN THE SOFTWARE.
- */
+// SPDX-License-Identifier: MIT OR LGPL-2.0-or-later
+// SPDX-FileCopyrightText: 2008 litl, LLC
 
 #ifndef GI_GERROR_H_
 #define GI_GERROR_H_
@@ -34,6 +15,7 @@
 #include <js/TypeDecls.h>
 
 #include "gi/wrapperutils.h"
+#include "gjs/jsapi-util.h"  // for GjsAutoPointer operators
 #include "gjs/macros.h"
 #include "util/log.h"
 
@@ -64,10 +46,9 @@ class ErrorBase
  protected:
     explicit ErrorBase(ErrorPrototype* proto = nullptr)
         : GIWrapperBase(proto) {}
-    ~ErrorBase(void) {}
 
-    static const GjsDebugTopic debug_topic = GJS_DEBUG_GERROR;
-    static constexpr const char* debug_tag = "gerror";
+    static constexpr GjsDebugTopic DEBUG_TOPIC = GJS_DEBUG_GERROR;
+    static constexpr const char* DEBUG_TAG = "gerror";
 
     static const struct JSClassOps class_ops;
     static const struct JSClass klass;
