@@ -1,26 +1,7 @@
 /* -*- mode: C++; c-basic-offset: 4; indent-tabs-mode: nil; -*- */
-/*
- * Copyright (c) 2008  litl, LLC
- * Copyright (c) 2012  Red Hat, Inc.
- *
- * Permission is hereby granted, free of charge, to any person obtaining a copy
- * of this software and associated documentation files (the "Software"), to
- * deal in the Software without restriction, including without limitation the
- * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
- * sell copies of the Software, and to permit persons to whom the Software is
- * furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice shall be included in
- * all copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
- * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
- * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
- * IN THE SOFTWARE.
- */
+// SPDX-License-Identifier: MIT OR LGPL-2.0-or-later
+// SPDX-FileCopyrightText: 2008 litl, LLC
+// SPDX-FileCopyrightText: 2012 Red Hat, Inc.
 
 #ifndef GI_INTERFACE_H_
 #define GI_INTERFACE_H_
@@ -64,17 +45,14 @@ class InterfaceBase : public GIWrapperBase<InterfaceBase, InterfacePrototype,
  protected:
     explicit InterfaceBase(InterfacePrototype* proto = nullptr)
         : GIWrapperBase(proto) {}
-    ~InterfaceBase(void) {}
 
-    static const GjsDebugTopic debug_topic = GJS_DEBUG_GINTERFACE;
-    static constexpr const char* debug_tag = "GInterface";
+    static constexpr GjsDebugTopic DEBUG_TOPIC = GJS_DEBUG_GINTERFACE;
+    static constexpr const char* DEBUG_TAG = "interface";
 
     static const struct JSClassOps class_ops;
     static const struct JSClass klass;
     static JSFunctionSpec static_methods[];
 
-    [[nodiscard]] const char* to_string_kind(void) const { return "interface"; }
-
     // JSNative methods
 
     // Overrides GIWrapperBase::constructor().
