@@ -1,25 +1,6 @@
 /* -*- mode: C++; c-basic-offset: 4; indent-tabs-mode: nil; -*- */
-/*
- * Copyright (c) 2008  litl, LLC
- *
- * Permission is hereby granted, free of charge, to any person obtaining a copy
- * of this software and associated documentation files (the "Software"), to
- * deal in the Software without restriction, including without limitation the
- * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
- * sell copies of the Software, and to permit persons to whom the Software is
- * furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice shall be included in
- * all copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
- * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
- * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
- * IN THE SOFTWARE.
- */
+// SPDX-License-Identifier: MIT OR LGPL-2.0-or-later
+// SPDX-FileCopyrightText: 2008 litl, LLC
 
 #include <config.h>
 
@@ -28,7 +9,9 @@
 
 #include <js/CallArgs.h>
 #include <js/Class.h>
+#include <js/ComparisonOperators.h>
 #include <js/Id.h>  // for JSID_IS_STRING
+#include <js/PropertyDescriptor.h>  // for JSPROP_READONLY
 #include <js/PropertySpec.h>
 #include <js/RootingAPI.h>
 #include <js/TypeDecls.h>
@@ -45,9 +28,10 @@
 #include "gjs/mem-private.h"
 #include "util/log.h"
 
-typedef struct {
-    char *gi_namespace;
-} Ns;
+struct Ns : GjsAutoChar {
+    explicit Ns(const char* ns_name)
+        : GjsAutoChar(const_cast<char*>(ns_name), GjsAutoTakeOwnership()) {}
+};
 
 extern struct JSClass gjs_ns_class;
 
@@ -96,7 +80,7 @@ ns_resolve(JSContext       *context,
     }
 
     GjsAutoBaseInfo info =
-        g_irepository_find_by_name(nullptr, priv->gi_namespace, name.get());
+        g_irepository_find_by_name(nullptr, priv->get(), name.get());
     if (!info) {
         *resolved = false; /* No property defined, but no error either */
         return true;
@@ -127,15 +111,14 @@ static bool ns_new_enumerate(JSContext* cx, JS::HandleObject obj,
         return true;
     }
 
-    int n = g_irepository_get_n_infos(nullptr, priv->gi_namespace);
+    int n = g_irepository_get_n_infos(nullptr, priv->get());
     if (!properties.reserve(properties.length() + n)) {
         JS_ReportOutOfMemory(cx);
         return false;
     }
 
     for (int k = 0; k < n; k++) {
-        GjsAutoBaseInfo info =
-            g_irepository_get_info(nullptr, priv->gi_namespace, k);
+        GjsAutoBaseInfo info = g_irepository_get_info(nullptr, priv->get(), k);
         const char* name = info.name();
 
         jsid id = gjs_intern_string_to_id(cx, name);
@@ -158,7 +141,7 @@ get_name (JSContext *context,
     if (!priv)
         return false;
 
-    return gjs_string_from_utf8(context, priv->gi_namespace, args.rval());
+    return gjs_string_from_utf8(context, priv->get(), args.rval());
 }
 
 GJS_NATIVE_CONSTRUCTOR_DEFINE_ABSTRACT(ns)
@@ -172,11 +155,8 @@ static void ns_finalize(JSFreeOp*, JSObject* obj) {
     if (!priv)
         return; /* we are the prototype, not a real instance */
 
-    if (priv->gi_namespace)
-        g_free(priv->gi_namespace);
-
     GJS_DEC_COUNTER(ns);
-    g_slice_free(Ns, priv);
+    delete priv;
 }
 
 /* The bizarre thing about this vtable is that it applies to both
@@ -216,8 +196,6 @@ static JSObject*
 ns_new(JSContext    *context,
        const char   *ns_name)
 {
-    Ns *priv;
-
     JS::RootedObject proto(context);
     if (!gjs_ns_define_proto(context, nullptr, &proto))
         return nullptr;
@@ -227,8 +205,7 @@ ns_new(JSContext    *context,
     if (!ns)
         return nullptr;
 
-    priv = g_slice_new0(Ns);
-
+    auto* priv = new Ns(ns_name);
     GJS_INC_COUNTER(ns);
 
     g_assert(!priv_from_js(context, ns));
@@ -237,8 +214,6 @@ ns_new(JSContext    *context,
     gjs_debug_lifecycle(GJS_DEBUG_GNAMESPACE, "ns constructor, obj %p priv %p",
                         ns.get(), priv);
 
-    priv = priv_from_js(context, ns);
-    priv->gi_namespace = g_strdup(ns_name);
     return ns;
 }
 
