@@ -1,29 +1,9 @@
 /* -*- mode: C++; c-basic-offset: 4; indent-tabs-mode: nil; -*- */
-/*
- * Copyright (c) 2008  litl, LLC
- *
- * Permission is hereby granted, free of charge, to any person obtaining a copy
- * of this software and associated documentation files (the "Software"), to
- * deal in the Software without restriction, including without limitation the
- * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
- * sell copies of the Software, and to permit persons to whom the Software is
- * furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice shall be included in
- * all copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
- * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
- * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
- * IN THE SOFTWARE.
- */
+// SPDX-License-Identifier: MIT OR LGPL-2.0-or-later
+// SPDX-FileCopyrightText: 2008 litl, LLC
 
 #include <config.h>
 
-#include <limits.h>  // for SCHAR_MAX, SCHAR_MIN, UCHAR_MAX
 #include <stdint.h>
 #include <string.h>  // for memset
 
@@ -50,6 +30,7 @@
 #include "gi/fundamental.h"
 #include "gi/gerror.h"
 #include "gi/gtype.h"
+#include "gi/js-value-inl.h"
 #include "gi/object.h"
 #include "gi/param.h"
 #include "gi/union.h"
@@ -143,9 +124,6 @@ closure_marshal(GClosure        *closure,
     unsigned i;
     GSignalQuery signal_query = { 0, };
     GISignalInfo *signal_info;
-    bool *skip;
-    int *array_len_indices_for;
-    GITypeInfo **type_info_for;
 
     gjs_debug_marshal(GJS_DEBUG_GCLOSURE,
                       "Marshal closure %p",
@@ -206,13 +184,12 @@ closure_marshal(GClosure        *closure,
     /* Check if any parameters, such as array lengths, need to be eliminated
      * before we invoke the closure.
      */
-    skip = g_newa(bool, n_param_values);
-    memset(skip, 0, sizeof (bool) * n_param_values);
-    array_len_indices_for = g_newa(int, n_param_values);
+    GjsAutoPointer<bool> skip = g_new0(bool, n_param_values);
+    GjsAutoPointer<int> array_len_indices_for = g_new(int, n_param_values);
     for(i = 0; i < n_param_values; i++)
         array_len_indices_for[i] = -1;
-    type_info_for = g_newa(GITypeInfo *, n_param_values);
-    memset(type_info_for, 0, sizeof (gpointer) * n_param_values);
+    GjsAutoPointer<GITypeInfo> type_info_for =
+        g_new0(GITypeInfo, n_param_values);
 
     signal_info = get_signal_info_if_available(&signal_query);
     if (signal_info) {
@@ -222,9 +199,9 @@ closure_marshal(GClosure        *closure,
             int array_len_pos;
 
             arg_info = g_callable_info_get_arg(signal_info, i - 1);
-            type_info_for[i] = g_arg_info_get_type(arg_info);
+            g_arg_info_load_type(arg_info, &type_info_for[i]);
 
-            array_len_pos = g_type_info_get_array_length(type_info_for[i]);
+            array_len_pos = g_type_info_get_array_length(&type_info_for[i]);
             if (array_len_pos != -1) {
                 skip[array_len_pos + 1] = true;
                 array_len_indices_for[i] = array_len_pos + 1;
@@ -259,12 +236,9 @@ closure_marshal(GClosure        *closure,
         array_len_index = array_len_indices_for[i];
         if (array_len_index != -1) {
             const GValue *array_len_gval = &param_values[array_len_index];
-            res = gjs_value_from_array_and_length_values(context,
-                                                         &argv_to_append,
-                                                         type_info_for[i],
-                                                         gval, array_len_gval,
-                                                         no_copy, &signal_query,
-                                                         array_len_index);
+            res = gjs_value_from_array_and_length_values(
+                context, &argv_to_append, &type_info_for[i], gval,
+                array_len_gval, no_copy, &signal_query, array_len_index);
         } else {
             res = gjs_value_from_g_value_internal(context,
                                                   &argv_to_append,
@@ -283,10 +257,6 @@ closure_marshal(GClosure        *closure,
         argv.infallibleAppend(argv_to_append);
     }
 
-    for (i = 1; i < n_param_values; i++)
-        if (type_info_for[i])
-            g_base_info_unref((GIBaseInfo *)type_info_for[i]);
-
     JS::RootedValue rval(context);
     mozilla::Unused << gjs_closure_invoke(closure, nullptr, argv, &rval, false);
     // Any exception now pending, is handled when returning control to JS
@@ -383,6 +353,7 @@ gjs_value_to_g_value_internal(JSContext      *context,
                               bool            no_copy)
 {
     GType gtype;
+    bool out_of_range = false;
 
     gtype = G_VALUE_TYPE(gvalue);
 
@@ -425,42 +396,48 @@ gjs_value_to_g_value_internal(JSContext      *context,
         }
     } else if (gtype == G_TYPE_CHAR) {
         gint32 i;
-        if (JS::ToInt32(context, value, &i) && i >= SCHAR_MIN && i <= SCHAR_MAX) {
+        if (Gjs::js_value_to_c_checked<signed char>(context, value, &i,
+                                                    &out_of_range) &&
+            !out_of_range) {
             g_value_set_schar(gvalue, (signed char)i);
         } else {
             return throw_expect_type(context, value, "char");
         }
     } else if (gtype == G_TYPE_UCHAR) {
-        guint16 i;
-        if (JS::ToUint16(context, value, &i) && i <= UCHAR_MAX) {
+        uint32_t i;
+        if (Gjs::js_value_to_c_checked<unsigned char>(context, value, &i,
+                                                      &out_of_range) &&
+            !out_of_range) {
             g_value_set_uchar(gvalue, (unsigned char)i);
         } else {
             return throw_expect_type(context, value, "unsigned char");
         }
     } else if (gtype == G_TYPE_INT) {
         gint32 i;
-        if (JS::ToInt32(context, value, &i)) {
+        if (Gjs::js_value_to_c(context, value, &i)) {
             g_value_set_int(gvalue, i);
         } else {
             return throw_expect_type(context, value, "integer");
         }
     } else if (gtype == G_TYPE_DOUBLE) {
         gdouble d;
-        if (JS::ToNumber(context, value, &d)) {
+        if (Gjs::js_value_to_c(context, value, &d)) {
             g_value_set_double(gvalue, d);
         } else {
             return throw_expect_type(context, value, "double");
         }
     } else if (gtype == G_TYPE_FLOAT) {
         gdouble d;
-        if (JS::ToNumber(context, value, &d)) {
+        if (Gjs::js_value_to_c_checked<float>(context, value, &d,
+                                              &out_of_range) &&
+            !out_of_range) {
             g_value_set_float(gvalue, d);
         } else {
             return throw_expect_type(context, value, "float");
         }
     } else if (gtype == G_TYPE_UINT) {
         guint32 i;
-        if (JS::ToUint32(context, value, &i)) {
+        if (Gjs::js_value_to_c(context, value, &i)) {
             g_value_set_uint(gvalue, i);
         } else {
             return throw_expect_type(context, value, "unsigned integer");
@@ -576,12 +553,10 @@ gjs_value_to_g_value_internal(JSContext      *context,
                         g_struct_info_is_foreign ((GIStructInfo*)registered)) {
                         GArgument arg;
 
-                        if (!gjs_struct_foreign_convert_to_g_argument (context, value,
-                                                                       registered,
-                                                                       NULL,
-                                                                       GJS_ARGUMENT_ARGUMENT,
-                                                                       GI_TRANSFER_NOTHING,
-                                                                       true, &arg))
+                        if (!gjs_struct_foreign_convert_to_g_argument(
+                                context, value, registered, nullptr,
+                                GJS_ARGUMENT_ARGUMENT, GI_TRANSFER_NOTHING,
+                                GjsArgumentFlags::MAY_BE_NULL, &arg))
                             return false;
 
                         gboxed = gjs_arg_get<void*>(&arg);
@@ -639,7 +614,7 @@ gjs_value_to_g_value_internal(JSContext      *context,
     } else if (g_type_is_a(gtype, G_TYPE_ENUM)) {
         int64_t value_int64;
 
-        if (JS::ToInt64(context, value, &value_int64)) {
+        if (Gjs::js_value_to_c(context, value, &value_int64)) {
             GEnumValue *v;
             GjsAutoTypeClass<GEnumClass> enum_class(gtype);
 
@@ -659,7 +634,7 @@ gjs_value_to_g_value_internal(JSContext      *context,
     } else if (g_type_is_a(gtype, G_TYPE_FLAGS)) {
         int64_t value_int64;
 
-        if (JS::ToInt64(context, value, &value_int64)) {
+        if (Gjs::js_value_to_c(context, value, &value_int64)) {
             if (!_gjs_flags_value_is_valid(context, gtype, value_int64))
                 return false;
 
@@ -711,7 +686,7 @@ gjs_value_to_g_value_internal(JSContext      *context,
          * e.g. ClutterUnit.
          */
         gint32 i;
-        if (JS::ToInt32(context, value, &i)) {
+        if (Gjs::js_value_to_c(context, value, &i)) {
             GValue int_value = { 0, };
             g_value_init(&int_value, G_TYPE_INT);
             g_value_set_int(&int_value, i);
