@@ -1,25 +1,6 @@
 /* -*- mode: C++; c-basic-offset: 4; indent-tabs-mode: nil; -*- */
-/*
- * Copyright (c) 2014 Colin Walters <walters@verbum.org>
- *
- * Permission is hereby granted, free of charge, to any person obtaining a copy
- * of this software and associated documentation files (the "Software"), to
- * deal in the Software without restriction, including without limitation the
- * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
- * sell copies of the Software, and to permit persons to whom the Software is
- * furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice shall be included in
- * all copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
- * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
- * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
- * IN THE SOFTWARE.
- */
+// SPDX-License-Identifier: MIT OR LGPL-2.0-or-later
+// SPDX-FileCopyrightText: 2014 Colin Walters <walters@verbum.org>
 
 #ifndef GJS_CONTEXT_PRIVATE_H_
 #define GJS_CONTEXT_PRIVATE_H_
@@ -36,15 +17,16 @@
 #include <glib.h>
 
 #include <js/GCHashTable.h>
+#include <js/GCPolicyAPI.h>
 #include <js/GCVector.h>
+#include <js/HashTable.h>  // for DefaultHasher
 #include <js/Promise.h>
 #include <js/RootingAPI.h>
 #include <js/TypeDecls.h>
+#include <js/UniquePtr.h>
 #include <js/ValueArray.h>
 #include <jsapi.h>        // for JS_GetContextPrivate
 #include <jsfriendapi.h>  // for ScriptEnvironmentPreparer
-#include <mozilla/HashTable.h>  // for DefaultHasher
-#include <mozilla/UniquePtr.h>
 
 #include "gjs/context.h"
 #include "gjs/jsapi-util.h"
@@ -75,11 +57,6 @@ using GTypeNotUint64 =
 // The GC sweep method should ignore FundamentalTable and GTypeTable's key types
 namespace JS {
 // Forward declarations
-template <typename T>
-class WeakCache;
-template <typename T>
-struct GCPolicy;
-
 template <>
 struct GCPolicy<void*> : public IgnoreGCPolicy<void*> {};
 // We need GCPolicy<GType> for GTypeTable. SpiderMonkey already defines
@@ -214,9 +191,9 @@ class GjsContextPrivate : public JS::JobQueue {
         return *(from_cx(cx)->m_atoms);
     }
 
-    GJS_JSAPI_RETURN_CONVENTION
-    bool eval(const char* script, ssize_t script_len, const char* filename,
-              int* exit_status_p, GError** error);
+    [[nodiscard]] bool eval(const char* script, ssize_t script_len,
+                            const char* filename, int* exit_status_p,
+                            GError** error);
     GJS_JSAPI_RETURN_CONVENTION
     bool eval_with_scope(JS::HandleObject scope_object, const char* script,
                          ssize_t script_len, const char* filename,
