@@ -1,4 +1,7 @@
 // -*- mode: js; indent-tabs-mode: nil -*-
+// SPDX-License-Identifier: MIT OR LGPL-2.0-or-later
+// SPDX-FileCopyrightText: 2011 Giovanni Campagna <gcampagna@src.gnome.org>
+
 imports.gi.versions.Gtk = '3.0';
 
 const Gio = imports.gi.Gio;
@@ -65,12 +68,7 @@ const MyObject = GObject.registerClass({
     }
 
     set construct(val) {
-        // this should be called at most once
-        if (this._constructCalled)
-            throw Error('Construct-Only property set more than once');
-
         this._constructProp = val;
-        this._constructCalled = true;
     }
 
     notifyProp() {
@@ -214,13 +212,17 @@ describe('GObject class with decorator', function () {
         expect(myInstance3.construct).toEqual('quz');
     });
 
+    it('does not allow changing CONSTRUCT_ONLY properties', function () {
+        myInstance.construct = 'val';
+        expect(myInstance.construct).toEqual('default');
+    });
+
     it('has a name', function () {
         expect(MyObject.name).toEqual('MyObject');
     });
 
     // the following would (should) cause a CRITICAL:
     // myInstance.readonly = 'val';
-    // myInstance.construct = 'val';
 
     it('has a notify signal', function () {
         let notifySpy = jasmine.createSpy('notifySpy');
