@@ -1,3 +1,7 @@
+// SPDX-License-Identifier: MIT OR LGPL-2.0-or-later
+// SPDX-FileCopyrightText: 2008 litl, LLC
+// SPDX-FileCopyrightText: 2008 Red Hat, Inc.
+
 const Regress = imports.gi.Regress;
 
 // We use Gio to have some objects that we know exist
@@ -847,7 +851,7 @@ describe('Life, the Universe and Everything', function () {
         });
 
         it('method cannot be called on a GBoxed of the wrong type', function () {
-            expect(() => Regress.TestSimpleBoxedA.protoype.copy.call(wrongBoxed))
+            expect(() => Regress.TestSimpleBoxedA.prototype.copy.call(wrongBoxed))
                 .toThrow();
         });
 
