@@ -1,24 +1,6 @@
 /* -*- mode: C++; c-basic-offset: 4; indent-tabs-mode: nil; -*- */
-/* Copyright 2010 litl, LLC.
- *
- * Permission is hereby granted, free of charge, to any person obtaining a copy
- * of this software and associated documentation files (the "Software"), to
- * deal in the Software without restriction, including without limitation the
- * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
- * sell copies of the Software, and to permit persons to whom the Software is
- * furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice shall be included in
- * all copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
- * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
- * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
- * IN THE SOFTWARE.
- */
+// SPDX-License-Identifier: MIT OR LGPL-2.0-or-later
+// SPDX-FileCopyrightText: 2010 litl, LLC.
 
 #include <config.h>
 
@@ -39,24 +21,32 @@
 #include <js/TypeDecls.h>
 #include <js/Utility.h>  // for UniqueChars
 #include <js/Value.h>
+#include <js/ValueArray.h>
 #include <jsapi.h>  // for JS_SetElement
 
 #include "gi/arg-inl.h"
 #include "gi/arg.h"
 #include "gi/foreign.h"
+#include "gjs/enum-utils.h"
 #include "gjs/jsapi-class.h"
 #include "gjs/jsapi-util-args.h"
 #include "gjs/jsapi-util.h"
 #include "gjs/macros.h"
 #include "modules/cairo-private.h"
 
-#define _GJS_CAIRO_CONTEXT_DEFINE_FUNC_BEGIN(mname)              \
-    GJS_JSAPI_RETURN_CONVENTION                                  \
-    static bool mname##_func(JSContext* context, unsigned argc,  \
-                             JS::Value* vp) {                    \
-        GJS_GET_PRIV(context, argc, vp, argv, obj, cairo_t, cr); \
-        if (!cr)                                                 \
-            return true;
+struct GjsCairoContext;
+
+#define _GJS_CAIRO_CONTEXT_GET_PRIV_CR_CHECKED(cx, argc, vp, argv, obj) \
+    GJS_GET_PRIV(context, argc, vp, argv, obj, GjsCairoContext, priv);  \
+    cairo_t* cr = priv->get();                                          \
+    if (!cr)                                                            \
+        return true;
+
+#define _GJS_CAIRO_CONTEXT_DEFINE_FUNC_BEGIN(mname)             \
+    GJS_JSAPI_RETURN_CONVENTION                                 \
+    static bool mname##_func(JSContext* context, unsigned argc, \
+                             JS::Value* vp) {                   \
+        _GJS_CAIRO_CONTEXT_GET_PRIV_CR_CHECKED(context, argc, vp, argv, obj)
 
 #define _GJS_CAIRO_CONTEXT_DEFINE_FUNC_END                               \
     return gjs_cairo_check_status(context, cairo_status(cr), "context"); \
@@ -260,14 +250,22 @@ _GJS_CAIRO_CONTEXT_DEFINE_FUNC_END
 
 [[nodiscard]] static JSObject* gjs_cairo_context_get_proto(JSContext*);
 
+struct GjsCairoContext
+    : GjsAutoPointer<cairo_t, cairo_t, cairo_destroy, cairo_reference> {
+    explicit GjsCairoContext(cairo_t* cr)
+        : GjsAutoPointer(cr, GjsAutoTakeOwnership()) {}
+};
+
 GJS_DEFINE_PROTO_WITH_GTYPE("Context", cairo_context,
                             CAIRO_GOBJECT_TYPE_CONTEXT,
                             JSCLASS_BACKGROUND_FINALIZE)
-GJS_DEFINE_PRIV_FROM_JS(cairo_t, gjs_cairo_context_class);
+GJS_DEFINE_PRIV_FROM_JS(GjsCairoContext, gjs_cairo_context_class);
 
-static void _gjs_cairo_context_construct_internal(JSObject* obj, cairo_t* cr) {
-    g_assert(!JS_GetPrivate(obj));
-    JS_SetPrivate(obj, cairo_reference(cr));
+static void _gjs_cairo_context_construct_internal(JSContext* context,
+                                                  JS::HandleObject obj,
+                                                  cairo_t* cr) {
+    g_assert(!priv_from_js(context, obj));
+    JS_SetPrivate(obj, new GjsCairoContext(cr));
 }
 
 GJS_NATIVE_CONSTRUCTOR_DECLARE(cairo_context)
@@ -292,7 +290,7 @@ GJS_NATIVE_CONSTRUCTOR_DECLARE(cairo_context)
     if (!gjs_cairo_check_status(context, cairo_status(cr), "context"))
         return false;
 
-    _gjs_cairo_context_construct_internal(object, cr);
+    _gjs_cairo_context_construct_internal(context, object, cr);
     cairo_destroy(cr);
 
     GJS_NATIVE_CONSTRUCTOR_FINISH(cairo_context);
@@ -301,8 +299,7 @@ GJS_NATIVE_CONSTRUCTOR_DECLARE(cairo_context)
 }
 
 static void gjs_cairo_context_finalize(JSFreeOp*, JSObject* obj) {
-    using AutoCairoContext = GjsAutoPointer<cairo_t, cairo_t, cairo_destroy>;
-    AutoCairoContext cr = static_cast<cairo_t*>(JS_GetPrivate(obj));
+    delete static_cast<GjsCairoContext*>(JS_GetPrivate(obj));
     JS_SetPrivate(obj, nullptr);
 }
 
@@ -398,9 +395,7 @@ dispose_func(JSContext *context,
              unsigned   argc,
              JS::Value *vp)
 {
-    GJS_GET_PRIV(context, argc, vp, rec, obj, cairo_t, cr);
-    if (!cr)
-        return true;
+    _GJS_CAIRO_CONTEXT_GET_PRIV_CR_CHECKED(context, argc, vp, rec, obj);
 
     cairo_destroy(cr);
     JS_SetPrivate(obj, nullptr);
@@ -415,9 +410,7 @@ appendPath_func(JSContext *context,
                 unsigned   argc,
                 JS::Value *vp)
 {
-    GJS_GET_PRIV(context, argc, vp, argv, obj, cairo_t, cr);
-    if (!cr)
-        return true;
+    _GJS_CAIRO_CONTEXT_GET_PRIV_CR_CHECKED(context, argc, vp, argv, obj);
 
     JS::RootedObject path_wrapper(context);
 
@@ -440,9 +433,7 @@ copyPath_func(JSContext *context,
               unsigned   argc,
               JS::Value *vp)
 {
-    GJS_GET_PRIV(context, argc, vp, argv, obj, cairo_t, cr);
-    if (!cr)
-        return true;
+    _GJS_CAIRO_CONTEXT_GET_PRIV_CR_CHECKED(context, argc, vp, argv, obj);
 
     cairo_path_t *path;
 
@@ -460,9 +451,7 @@ copyPathFlat_func(JSContext *context,
                   unsigned   argc,
                   JS::Value *vp)
 {
-    GJS_GET_PRIV(context, argc, vp, argv, obj, cairo_t, cr);
-    if (!cr)
-        return true;
+    _GJS_CAIRO_CONTEXT_GET_PRIV_CR_CHECKED(context, argc, vp, argv, obj);
 
     cairo_path_t *path;
 
@@ -480,9 +469,7 @@ mask_func(JSContext *context,
           unsigned   argc,
           JS::Value *vp)
 {
-    GJS_GET_PRIV(context, argc, vp, argv, obj, cairo_t, cr);
-    if (!cr)
-        return true;
+    _GJS_CAIRO_CONTEXT_GET_PRIV_CR_CHECKED(context, argc, vp, argv, obj);
 
     JS::RootedObject pattern_wrapper(context);
 
@@ -510,9 +497,7 @@ maskSurface_func(JSContext *context,
                  unsigned   argc,
                  JS::Value *vp)
 {
-    GJS_GET_PRIV(context, argc, vp, argv, obj, cairo_t, cr);
-    if (!cr)
-        return true;
+    _GJS_CAIRO_CONTEXT_GET_PRIV_CR_CHECKED(context, argc, vp, argv, obj);
 
     JS::RootedObject surface_wrapper(context);
     double x, y;
@@ -543,9 +528,7 @@ setDash_func(JSContext *context,
              unsigned   argc,
              JS::Value *vp)
 {
-    GJS_GET_PRIV(context, argc, vp, argv, obj, cairo_t, cr);
-    if (!cr)
-        return true;
+    _GJS_CAIRO_CONTEXT_GET_PRIV_CR_CHECKED(context, argc, vp, argv, obj);
 
     guint i;
     JS::RootedObject dashes(context);
@@ -604,9 +587,7 @@ setSource_func(JSContext *context,
                unsigned   argc,
                JS::Value *vp)
 {
-    GJS_GET_PRIV(context, argc, vp, argv, obj, cairo_t, cr);
-    if (!cr)
-        return true;
+    _GJS_CAIRO_CONTEXT_GET_PRIV_CR_CHECKED(context, argc, vp, argv, obj);
 
     JS::RootedObject pattern_wrapper(context);
 
@@ -635,9 +616,7 @@ setSourceSurface_func(JSContext *context,
                       unsigned   argc,
                       JS::Value *vp)
 {
-    GJS_GET_PRIV(context, argc, vp, argv, obj, cairo_t, cr);
-    if (!cr)
-        return true;
+    _GJS_CAIRO_CONTEXT_GET_PRIV_CR_CHECKED(context, argc, vp, argv, obj);
 
     JS::RootedObject surface_wrapper(context);
     double x, y;
@@ -669,9 +648,7 @@ showText_func(JSContext *context,
               unsigned   argc,
               JS::Value *vp)
 {
-    GJS_GET_PRIV(context, argc, vp, argv, obj, cairo_t, cr);
-    if (!cr)
-        return true;
+    _GJS_CAIRO_CONTEXT_GET_PRIV_CR_CHECKED(context, argc, vp, argv, obj);
 
     JS::UniqueChars utf8;
 
@@ -695,9 +672,7 @@ selectFontFace_func(JSContext *context,
                     unsigned   argc,
                     JS::Value *vp)
 {
-    GJS_GET_PRIV(context, argc, vp, argv, obj, cairo_t, cr);
-    if (!cr)
-        return true;
+    _GJS_CAIRO_CONTEXT_GET_PRIV_CR_CHECKED(context, argc, vp, argv, obj);
 
     JS::UniqueChars family;
     cairo_font_slant_t slant;
@@ -724,9 +699,7 @@ popGroup_func(JSContext *context,
               unsigned   argc,
               JS::Value *vp)
 {
-    GJS_GET_PRIV(context, argc, vp, rec, obj, cairo_t, cr);
-    if (!cr)
-        return true;
+    _GJS_CAIRO_CONTEXT_GET_PRIV_CR_CHECKED(context, argc, vp, rec, obj);
 
     cairo_pattern_t *pattern;
     JSObject *pattern_wrapper;
@@ -757,9 +730,7 @@ getSource_func(JSContext *context,
                unsigned   argc,
                JS::Value *vp)
 {
-    GJS_GET_PRIV(context, argc, vp, rec, obj, cairo_t, cr);
-    if (!cr)
-        return true;
+    _GJS_CAIRO_CONTEXT_GET_PRIV_CR_CHECKED(context, argc, vp, rec, obj);
 
     cairo_pattern_t *pattern;
     JSObject *pattern_wrapper;
@@ -791,9 +762,7 @@ getTarget_func(JSContext *context,
                unsigned   argc,
                JS::Value *vp)
 {
-    GJS_GET_PRIV(context, argc, vp, rec, obj, cairo_t, cr);
-    if (!cr)
-        return true;
+    _GJS_CAIRO_CONTEXT_GET_PRIV_CR_CHECKED(context, argc, vp, rec, obj);
 
     cairo_surface_t *surface;
     JSObject *surface_wrapper;
@@ -825,9 +794,7 @@ getGroupTarget_func(JSContext *context,
                     unsigned   argc,
                     JS::Value *vp)
 {
-    GJS_GET_PRIV(context, argc, vp, rec, obj, cairo_t, cr);
-    if (!cr)
-        return true;
+    _GJS_CAIRO_CONTEXT_GET_PRIV_CR_CHECKED(context, argc, vp, rec, obj);
 
     cairo_surface_t *surface;
     JSObject *surface_wrapper;
@@ -966,7 +933,7 @@ gjs_cairo_context_from_context(JSContext *context,
     if (!object)
         return nullptr;
 
-    _gjs_cairo_context_construct_internal(object, cr);
+    _gjs_cairo_context_construct_internal(context, object, cr);
 
     return object;
 }
@@ -975,15 +942,16 @@ cairo_t *
 gjs_cairo_context_get_context(JSContext       *context,
                               JS::HandleObject object)
 {
-    return priv_from_js(context, object);
+    auto* priv = priv_from_js(context, object);
+    return priv ? priv->get() : nullptr;
 }
 
 [[nodiscard]] static bool context_to_g_argument(
     JSContext* context, JS::Value value, const char* arg_name,
-    GjsArgumentType argument_type, GITransfer transfer, bool may_be_null,
+    GjsArgumentType argument_type, GITransfer transfer, GjsArgumentFlags flags,
     GIArgument* arg) {
     if (value.isNull()) {
-        if (!may_be_null) {
+        if (!(flags & GjsArgumentFlags::MAY_BE_NULL)) {
             GjsAutoChar display_name =
                 gjs_argument_display_name(arg_name, argument_type);
             gjs_throw(context, "%s may not be null", display_name.get());
