@@ -1,24 +1,6 @@
 /* -*- mode: C++; c-basic-offset: 4; indent-tabs-mode: nil; -*- */
-/* Copyright 2010 Red Hat, Inc.
- *
- * Permission is hereby granted, free of charge, to any person obtaining a copy
- * of this software and associated documentation files (the "Software"), to
- * deal in the Software without restriction, including without limitation the
- * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
- * sell copies of the Software, and to permit persons to whom the Software is
- * furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice shall be included in
- * all copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
- * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
- * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
- * IN THE SOFTWARE.
- */
+// SPDX-License-Identifier: MIT OR LGPL-2.0-or-later
+// SPDX-FileCopyrightText: 2010 Red Hat, Inc.
 
 #include <config.h>
 
@@ -37,12 +19,16 @@
 
 [[nodiscard]] static JSObject* gjs_cairo_path_get_proto(JSContext*);
 
+struct GjsCairoPath
+    : GjsAutoPointer<cairo_path_t, cairo_path_t, cairo_path_destroy> {
+    explicit GjsCairoPath(cairo_path_t* path) : GjsAutoPointer(path) {}
+};
+
 GJS_DEFINE_PROTO_ABSTRACT("Path", cairo_path, JSCLASS_BACKGROUND_FINALIZE)
+GJS_DEFINE_PRIV_FROM_JS(GjsCairoPath, gjs_cairo_path_class);
 
 static void gjs_cairo_path_finalize(JSFreeOp*, JSObject* obj) {
-    using AutoCairoPath =
-        GjsAutoPointer<cairo_path_t, cairo_path_t, cairo_path_destroy>;
-    AutoCairoPath path = static_cast<cairo_path_t*>(JS_GetPrivate(obj));
+    delete static_cast<GjsCairoPath*>(JS_GetPrivate(obj));
     JS_SetPrivate(obj, nullptr);
 }
 
@@ -82,8 +68,8 @@ gjs_cairo_path_from_path(JSContext    *context,
         return nullptr;
     }
 
-    g_assert(!JS_GetPrivate(object));
-    JS_SetPrivate(object, path);
+    g_assert(!priv_from_js(context, object));
+    JS_SetPrivate(object, new GjsCairoPath(path));
 
     return object;
 }
@@ -100,13 +86,12 @@ cairo_path_t* gjs_cairo_path_get_path(JSContext* cx,
     g_return_val_if_fail(cx, nullptr);
     g_return_val_if_fail(path_wrapper, nullptr);
 
-    auto* path = static_cast<cairo_path_t*>(JS_GetInstancePrivate(
-        cx, path_wrapper, &gjs_cairo_path_class, nullptr));
-    if (!path) {
+    GjsCairoPath* priv;
+    if (!priv_from_js_with_typecheck(cx, path_wrapper, &priv)) {
         gjs_throw(cx, "Expected Cairo.Path but got %s",
                   JS_GetClass(path_wrapper)->name);
         return nullptr;
     }
 
-    return path;
+    return priv->get();
 }
