@@ -1,24 +1,6 @@
 /* -*- mode: C++; c-basic-offset: 4; indent-tabs-mode: nil; -*- */
-/* Copyright 2010 litl, LLC.
- *
- * Permission is hereby granted, free of charge, to any person obtaining a copy
- * of this software and associated documentation files (the "Software"), to
- * deal in the Software without restriction, including without limitation the
- * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
- * sell copies of the Software, and to permit persons to whom the Software is
- * furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice shall be included in
- * all copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
- * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
- * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
- * IN THE SOFTWARE.
- */
+// SPDX-License-Identifier: MIT OR LGPL-2.0-or-later
+// SPDX-FileCopyrightText: 2010 litl, LLC.
 
 #include <config.h>
 
@@ -39,14 +21,19 @@
 #include "gjs/macros.h"
 #include "modules/cairo-private.h"
 
+struct GjsCairoPattern
+    : GjsAutoPointer<cairo_pattern_t, cairo_pattern_t, cairo_pattern_destroy,
+                     cairo_pattern_reference> {
+    explicit GjsCairoPattern(cairo_pattern_t* pattern)
+        : GjsAutoPointer(pattern, GjsAutoTakeOwnership()) {}
+};
+
 GJS_DEFINE_PROTO_ABSTRACT_WITH_GTYPE("Pattern", cairo_pattern,
                                      CAIRO_GOBJECT_TYPE_PATTERN,
                                      JSCLASS_BACKGROUND_FINALIZE)
 
 static void gjs_cairo_pattern_finalize(JSFreeOp*, JSObject* obj) {
-    using AutoPattern =
-        GjsAutoPointer<cairo_pattern_t, cairo_pattern_t, cairo_pattern_destroy>;
-    AutoPattern pattern = static_cast<cairo_pattern_t*>(JS_GetPrivate(obj));
+    delete static_cast<GjsCairoPattern*>(JS_GetPrivate(obj));
     JS_SetPrivate(obj, nullptr);
 }
 
@@ -108,7 +95,7 @@ void gjs_cairo_pattern_construct(JSObject* object, cairo_pattern_t* pattern) {
     g_return_if_fail(pattern);
 
     g_assert(!JS_GetPrivate(object));
-    JS_SetPrivate(object, cairo_pattern_reference(pattern));
+    JS_SetPrivate(object, new GjsCairoPattern(pattern));
 }
 
 /**
@@ -190,5 +177,6 @@ cairo_pattern_t* gjs_cairo_pattern_get_pattern(
         return nullptr;
     }
 
-    return static_cast<cairo_pattern_t*>(JS_GetPrivate(pattern_wrapper));
+    auto* priv = static_cast<GjsCairoPattern*>(JS_GetPrivate(pattern_wrapper));
+    return priv ? priv->get() : nullptr;
 }
