@@ -1,24 +1,6 @@
 /* -*- mode: C++; c-basic-offset: 4; indent-tabs-mode: nil; -*- */
-/* Copyright 2010 litl, LLC.
- *
- * Permission is hereby granted, free of charge, to any person obtaining a copy
- * of this software and associated documentation files (the "Software"), to
- * deal in the Software without restriction, including without limitation the
- * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
- * sell copies of the Software, and to permit persons to whom the Software is
- * furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice shall be included in
- * all copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
- * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
- * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
- * IN THE SOFTWARE.
- */
+// SPDX-License-Identifier: MIT OR LGPL-2.0-or-later
+// SPDX-FileCopyrightText: 2010 litl, LLC.
 
 #include <config.h>
 
@@ -39,20 +21,26 @@
 #include "gi/arg-inl.h"
 #include "gi/arg.h"
 #include "gi/foreign.h"
+#include "gjs/enum-utils.h"
 #include "gjs/jsapi-class.h"
 #include "gjs/jsapi-util-args.h"
 #include "gjs/jsapi-util.h"
 #include "gjs/macros.h"
 #include "modules/cairo-private.h"
 
+struct GjsCairoSurface
+    : GjsAutoPointer<cairo_surface_t, cairo_surface_t, cairo_surface_destroy,
+                     cairo_surface_reference> {
+    explicit GjsCairoSurface(cairo_surface_t* surface)
+        : GjsAutoPointer(surface, GjsAutoTakeOwnership()) {}
+};
+
 GJS_DEFINE_PROTO_ABSTRACT_WITH_GTYPE("Surface", cairo_surface,
                                      CAIRO_GOBJECT_TYPE_SURFACE,
                                      JSCLASS_BACKGROUND_FINALIZE)
 
 static void gjs_cairo_surface_finalize(JSFreeOp*, JSObject* obj) {
-    using AutoSurface =
-        GjsAutoPointer<cairo_surface_t, cairo_surface_t, cairo_surface_destroy>;
-    AutoSurface surface = static_cast<cairo_surface_t*>(JS_GetPrivate(obj));
+    delete static_cast<GjsCairoSurface*>(JS_GetPrivate(obj));
     JS_SetPrivate(obj, nullptr);
 }
 
@@ -152,7 +140,7 @@ void gjs_cairo_surface_construct(JSObject* object, cairo_surface_t* surface) {
     g_return_if_fail(surface);
 
     g_assert(!JS_GetPrivate(object));
-    JS_SetPrivate(object, cairo_surface_reference(surface));
+    JS_SetPrivate(object, new GjsCairoSurface(surface));
 }
 
 /**
@@ -237,15 +225,16 @@ cairo_surface_t* gjs_cairo_surface_get_surface(
         return nullptr;
     }
 
-    return static_cast<cairo_surface_t*>(JS_GetPrivate(surface_wrapper));
+    auto* priv = static_cast<GjsCairoSurface*>(JS_GetPrivate(surface_wrapper));
+    return priv ? priv->get() : nullptr;
 }
 
 [[nodiscard]] static bool surface_to_g_argument(
     JSContext* context, JS::Value value, const char* arg_name,
-    GjsArgumentType argument_type, GITransfer transfer, bool may_be_null,
+    GjsArgumentType argument_type, GITransfer transfer, GjsArgumentFlags flags,
     GIArgument* arg) {
     if (value.isNull()) {
-        if (!may_be_null) {
+        if (!(flags & GjsArgumentFlags::MAY_BE_NULL)) {
             GjsAutoChar display_name =
                 gjs_argument_display_name(arg_name, argument_type);
             gjs_throw(context, "%s may not be null", display_name.get());
