@@ -1,42 +1,7 @@
 /* -*- mode: C++; c-basic-offset: 4; indent-tabs-mode: nil; -*- */
-/* vim: set ts=8 sw=4 et tw=78:
- *
- * ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is Mozilla Communicator client code, released
- * March 31, 1998.
- *
- * The Initial Developer of the Original Code is
- * Netscape Communications Corporation.
- * Portions created by the Initial Developer are Copyright (C) 1998
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either of the GNU General Public License Version 2 or later (the "GPL"),
- * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* vim: set ts=8 sw=4 et tw=78: */
+// SPDX-License-Identifier: MPL-1.1 OR GPL-2.0-or-later OR LGPL-2.1-or-later
+// SPDX-FileCopyrightText: 1998 Netscape Communications Corporation
 
 #include <config.h>  // for HAVE_READLINE_READLINE_H
 
@@ -47,6 +12,8 @@
 #    include <readline/readline.h>
 #endif
 
+#include <string>
+
 #include <glib.h>
 #include <glib/gprintf.h>  // for g_fprintf
 
@@ -139,11 +106,11 @@ public:
  * invocation of this function.)
  */
 [[nodiscard]] static bool gjs_console_eval_and_print(JSContext* cx,
-                                                     const char* bytes,
-                                                     size_t length,
+                                                     const std::string& bytes,
                                                      int lineno) {
     JS::SourceText<mozilla::Utf8Unit> source;
-    if (!source.init(cx, bytes, length, JS::SourceOwnership::Borrowed))
+    if (!source.init(cx, bytes.c_str(), bytes.size(),
+                     JS::SourceOwnership::Borrowed))
         return false;
 
     JS::CompileOptions options(cx);
@@ -161,12 +128,7 @@ public:
     if (result.isUndefined())
         return true;
 
-    char *display_str;
-    display_str = gjs_value_debug_string(cx, result);
-    if (display_str) {
-        g_fprintf(stdout, "%s\n", display_str);
-        g_free(display_str);
-    }
+    g_fprintf(stdout, "%s\n", gjs_value_debug_string(cx, result).c_str());
     return true;
 }
 
@@ -179,7 +141,6 @@ gjs_console_interact(JSContext *context,
     JS::CallArgs argv = JS::CallArgsFromVp(argc, vp);
     bool eof = false;
     JS::RootedObject global(context, gjs_get_import_global(context));
-    GString *buffer = NULL;
     char *temp_buf = NULL;
     int lineno;
     int startline;
@@ -196,26 +157,24 @@ gjs_console_interact(JSContext *context,
          * coincides with the end of a line.
          */
         startline = lineno;
-        buffer = g_string_new("");
+        std::string buffer;
         do {
             if (!gjs_console_readline(
                     &temp_buf, startline == lineno ? "gjs> " : ".... ")) {
                 eof = true;
                 break;
             }
-            g_string_append(buffer, temp_buf);
+            buffer += temp_buf;
             g_free(temp_buf);
             lineno++;
-        } while (!JS_Utf8BufferIsCompilableUnit(context, global, buffer->str,
-                                                buffer->len));
+        } while (!JS_Utf8BufferIsCompilableUnit(context, global, buffer.c_str(),
+                                                buffer.size()));
 
         bool ok;
         {
             AutoReportException are(context);
-            ok = gjs_console_eval_and_print(context, buffer->str, buffer->len,
-                                            startline);
+            ok = gjs_console_eval_and_print(context, buffer, startline);
         }
-        g_string_free(buffer, true);
 
         GjsContextPrivate* gjs = GjsContextPrivate::from_cx(context);
         ok = gjs->run_jobs_fallible() && ok;
