@@ -1,10 +1,7 @@
-/* global debuggee, quit, readline, uneval */
-/* -*- indent-tabs-mode: nil; js-indent-level: 4 -*-
- *
- * This Source Code Form is subject to the terms of the Mozilla Public
- * License, v. 2.0. If a copy of the MPL was not distributed with this
- * file, You can obtain one at http://mozilla.org/MPL/2.0/.
- */
+/* -*- indent-tabs-mode: nil; js-indent-level: 4 -*- */
+/* global debuggee, quit, loadNative, readline, uneval */
+// SPDX-License-Identifier: MPL-2.0
+// SPDX-FileCopyrightText: 2011 Mozilla Foundation and contributors
 
 /*
  * This is a simple command-line debugger for GJS programs. It is based on
@@ -16,7 +13,7 @@
  * the first frame is executed.
  */
 
-const {print, logError} = imports._print;
+const {print, logError} = loadNative('_print');
 
 // Debugger state.
 var focusedFrame = null;
@@ -82,6 +79,7 @@ function debuggeeValueToString(dv, style = {pretty: options.pretty}) {
 function showDebuggeeValue(dv, style = {pretty: options.pretty}) {
     const i = nextDebuggeeValueIndex++;
     debuggeeValues[`$${i}`] = dv;
+    debuggeeValues['$$'] = dv;
     const [brief, full] = debuggeeValueToString(dv, style);
     print(`$${i} = ${brief}`);
     if (full !== undefined)
@@ -142,7 +140,7 @@ Debugger.Script.prototype.describeOffset = function describeOffset(offset) {
     return `${url}:${lineNumber}:${columnNumber}`;
 };
 
-function showFrame(f, n) {
+function showFrame(f, n, option) {
     if (f === undefined || f === null) {
         f = focusedFrame;
         if (f === null) {
@@ -155,10 +153,23 @@ function showFrame(f, n) {
         if (n === undefined)
             throw new Error('Internal error: frame not on stack');
     }
+    if (!option) {
+        print(`#${n.toString().padEnd(4)} ${f.describeFull()}`);
+    } else {
+        const variables = f.environment.names();
+        print(`#${n.toString().padEnd(4)} ${f.describeFull()}`);
+        for (let i = 0; i < variables.length; i++) {
+            if (variables.length === 0)
+                print('No locals.');
 
-    print(`#${n.toString().padEnd(4)} ${f.describeFull()}`);
+            const value = f.environment.getVariable(variables[i]);
+            const [brief] = debuggeeValueToString(value, {brief: false, pretty: false});
+            print(`${variables[i]} = ${brief}`);
+        }
+    }
 }
 
+
 function saveExcursion(fn) {
     const tf = topFrame, ff = focusedFrame;
     try {
@@ -182,22 +193,33 @@ function evalCommand(expr) {
 }
 
 function quitCommand() {
-    dbg.enabled = false;
+    dbg.removeAllDebuggees();
     quit(0);
 }
 quitCommand.summary = 'Quit the debugger';
 quitCommand.helpText = `USAGE
     quit`;
 
-function backtraceCommand() {
+function backtraceCommand(option) {
     if (topFrame === null)
         print('No stack.');
-    for (var i = 0, f = topFrame; f; i++, f = f.older)
-        showFrame(f, i);
+    if (option === '') {
+        for (let i = 0, f = topFrame; f; i++, f = f.older)
+            showFrame(f, i, false);
+    } else if (option === 'full') {
+        for (let i = 0, f = topFrame; f; i++, f = f.older)
+            showFrame(f, i, true);
+    } else {
+        print('Invalid option');
+    }
 }
-backtraceCommand.summary = 'Print backtrace of all stack frames';
+backtraceCommand.summary = 'Print backtrace of all stack frames and details of all local variables if the full option is added';
 backtraceCommand.helpText = `USAGE
-    bt`;
+    bt <option>
+
+PARAMETERS
+    · option: option name. Allowed options are:
+        · full: prints the local variables in a stack frame`;
 
 function setCommand(rest) {
     var space = rest.indexOf(' ');
@@ -245,16 +267,10 @@ function doPrint(expr, style) {
             ? debuggeeGlobalWrapper.executeInGlobalWithBindings(expr, debuggeeValues)
             : focusedFrame.evalWithBindings(expr, debuggeeValues));
     if (cv === null) {
-        if (!dbg.enabled)
-            return [cv];
         print('Debuggee died.');
     } else if ('return' in cv) {
-        if (!dbg.enabled)
-            return [undefined];
         showDebuggeeValue(cv['return'], style);
     } else {
-        if (!dbg.enabled)
-            return [cv];
         print("Exception caught. (To rethrow it, type 'throw'.)");
         lastExc = cv.throw;
         showDebuggeeValue(lastExc, style);
@@ -272,7 +288,10 @@ printCommand.helpText = `USAGE
 PARAMETER
     · expr: expression to be printed
     · pretty|p: prettify the output
-    · brief|b: brief output`;
+    · brief|b: brief output
+
+expr may also reference the variables $1, $2, ... for already printed
+expressions, or $$ for the most recently printed expression.`;
 
 function keysCommand(rest) {
     return doPrint(`Object.keys(${rest})`);
@@ -285,7 +304,7 @@ PARAMETER
     · obj: object to get keys of`;
 
 function detachCommand() {
-    dbg.enabled = false;
+    dbg.removeAllDebuggees();
     return [undefined];
 }
 detachCommand.summary = 'Detach debugger from the script';
@@ -317,15 +336,11 @@ function throwOrReturn(rest, action, defaultCompletion) {
 
     const cv = saveExcursion(() => focusedFrame.eval(rest));
     if (cv === null) {
-        if (!dbg.enabled)
-            return [cv];
         print(`Debuggee died while determining what to ${action}. Stopped.`);
         return;
     }
     if ('return' in cv)
         return [{[action]: cv['return']}];
-    if (!dbg.enabled)
-        return [cv];
     print(`Exception determining what to ${action}. Stopped.`);
     showDebuggeeValue(cv.throw);
 }
