@@ -1,33 +1,18 @@
 /* -*- mode: C++; c-basic-offset: 4; indent-tabs-mode: nil; -*- */
-/*
- * Copyright (c) 2008  litl, LLC
- *
- * Permission is hereby granted, free of charge, to any person obtaining a copy
- * of this software and associated documentation files (the "Software"), to
- * deal in the Software without restriction, including without limitation the
- * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
- * sell copies of the Software, and to permit persons to whom the Software is
- * furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice shall be included in
- * all copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
- * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
- * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
- * IN THE SOFTWARE.
- */
+// SPDX-License-Identifier: MIT OR LGPL-2.0-or-later
+// SPDX-FileCopyrightText: 2008 litl, LLC
 
 #include <config.h>
 
 #include <stdint.h>
 #include <string.h>  // for size_t, strlen
 
+#include <limits>
+#include <random>
 #include <string>  // for u16string, u32string
+#include <type_traits>
 
+#include <girepository.h>
 #include <glib-object.h>
 #include <glib.h>
 #include <glib/gstdio.h>  // for g_unlink
@@ -59,6 +44,46 @@ void g_assertion_message(const char*, const char*, int, const char*,
 
 #define VALID_UTF8_STRING "\303\211\303\226 foobar \343\203\237"
 
+static unsigned cpp_random_seed = 0;
+
+template <typename T>
+T get_random_number() {
+    std::mt19937_64 gen(cpp_random_seed);
+
+    if constexpr (std::is_same_v<T, bool>) {
+        return g_random_boolean();
+    } else if constexpr (std::is_integral_v<T>) {
+        T lowest_value = std::numeric_limits<T>::lowest();
+
+        if constexpr (std::is_unsigned_v<T>)
+            lowest_value = 1;
+
+        return std::uniform_int_distribution<T>(lowest_value)(gen);
+    } else if constexpr (std::is_arithmetic_v<T>) {
+        T lowest_value = std::numeric_limits<T>::epsilon();
+        return std::uniform_real_distribution<T>(lowest_value)(gen);
+    } else if constexpr (std::is_pointer_v<T>) {
+        return reinterpret_cast<T>(get_random_number<uintptr_t>());
+    }
+}
+
+template <typename T>
+constexpr void assert_equal(T a, T b) {
+    if constexpr (std::is_integral_v<T>) {
+        if constexpr (std::is_unsigned_v<T>)
+            g_assert_cmpuint(a, ==, b);
+        else
+            g_assert_cmpint(a, ==, b);
+    } else if constexpr (std::is_arithmetic_v<T>) {
+        g_assert_cmpfloat_with_epsilon(a, b, std::numeric_limits<T>::epsilon());
+    } else if constexpr (std::is_same_v<T, char*>) {
+        g_assert_cmpstr(a, ==, b);
+    } else if constexpr (std::is_pointer_v<T>) {
+        assert_equal(reinterpret_cast<uintptr_t>(a),
+                     reinterpret_cast<uintptr_t>(b));
+    }
+}
+
 static void
 gjstest_test_func_gjs_context_construct_destroy(void)
 {
@@ -256,7 +281,8 @@ static void test_jsapi_util_string_char16_data(GjsUnitTestFixture* fx,
 
     /* Try with a string that is likely to be stored as Latin-1 */
     str = JS_NewStringCopyZ(fx->cx, "abcd");
-    g_assert_true(gjs_string_get_char16_data(fx->cx, str, &chars, &len));
+    bool ok = gjs_string_get_char16_data(fx->cx, str, &chars, &len);
+    g_assert_true(ok);
 
     result.assign(chars, len);
     g_assert_true(result == u"abcd");
@@ -278,7 +304,8 @@ static void test_jsapi_util_string_to_ucs4(GjsUnitTestFixture* fx,
 
     /* Try with a string that is likely to be stored as Latin-1 */
     str = JS_NewStringCopyZ(fx->cx, "abcd");
-    g_assert_true(gjs_string_to_ucs4(fx->cx, str, &chars, &len));
+    bool ok = gjs_string_to_ucs4(fx->cx, str, &chars, &len);
+    g_assert_true(ok);
 
     result.assign(chars, chars + len);
     g_assert_true(result == U"abcd");
@@ -290,12 +317,9 @@ static void test_jsapi_util_debug_string_valid_utf8(GjsUnitTestFixture* fx,
     JS::RootedValue v_string(fx->cx);
     g_assert_true(gjs_string_from_utf8(fx->cx, VALID_UTF8_STRING, &v_string));
 
-    char *debug_output = gjs_value_debug_string(fx->cx, v_string);
+    std::string debug_output = gjs_value_debug_string(fx->cx, v_string);
 
-    g_assert_nonnull(debug_output);
-    g_assert_cmpstr("\"" VALID_UTF8_STRING "\"", ==, debug_output);
-
-    g_free(debug_output);
+    g_assert_cmpstr("\"" VALID_UTF8_STRING "\"", ==, debug_output.c_str());
 }
 
 static void test_jsapi_util_debug_string_invalid_utf8(GjsUnitTestFixture* fx,
@@ -306,12 +330,8 @@ static void test_jsapi_util_debug_string_invalid_utf8(GjsUnitTestFixture* fx,
     const char16_t invalid_unicode[] = { 0xffff, 0xffff };
     v_string.setString(JS_NewUCStringCopyN(fx->cx, invalid_unicode, 2));
 
-    char *debug_output = gjs_value_debug_string(fx->cx, v_string);
-
-    g_assert_nonnull(debug_output);
-    /* g_assert_cmpstr("\"\\xff\\xff\\xff\\xff\"", ==, debug_output); */
-
-    g_free(debug_output);
+    std::string debug_output = gjs_value_debug_string(fx->cx, v_string);
+    // g_assert_cmpstr("\"\\xff\\xff\\xff\\xff\"", ==, debug_output.c_str());
 }
 
 static void test_jsapi_util_debug_string_object_with_complicated_to_string(
@@ -325,12 +345,9 @@ static void test_jsapi_util_debug_string_object_with_complicated_to_string(
     contents[1].setString(JS_NewUCStringCopyN(fx->cx, desserts + 2, 2));
     JS::RootedObject array(fx->cx, JS::NewArrayObject(fx->cx, contents));
     JS::RootedValue v_array(fx->cx, JS::ObjectValue(*array));
-    char *debug_output = gjs_value_debug_string(fx->cx, v_array);
+    std::string debug_output = gjs_value_debug_string(fx->cx, v_array);
 
-    g_assert_nonnull(debug_output);
-    g_assert_cmpstr(u8"🍪,🍩", ==, debug_output);
-
-    g_free(debug_output);
+    g_assert_cmpstr(u8"🍪,🍩", ==, debug_output.c_str());
 }
 
 static void
@@ -425,6 +442,151 @@ static void gjstest_test_safe_integer_min(GjsUnitTestFixture* fx, const void*) {
     g_assert_cmpint(safe_value.toNumber(), ==, min_safe_big_number<int64_t>());
 }
 
+static void gjstest_test_args_set_get_unset() {
+    GIArgument arg = {0};
+
+    gjs_arg_set(&arg, true);
+    g_assert_true(arg.v_boolean);
+
+    gjs_arg_set(&arg, false);
+    g_assert_false(arg.v_boolean);
+
+    gjs_arg_set(&arg, true);
+    g_assert_true(arg.v_boolean);
+    gjs_arg_unset<bool>(&arg);
+    g_assert_false(arg.v_boolean);
+
+    int8_t random_int8 = get_random_number<int8_t>();
+    gjs_arg_set(&arg, random_int8);
+    assert_equal(arg.v_int8, random_int8);
+    assert_equal(gjs_arg_get<int8_t>(&arg), random_int8);
+
+    uint8_t random_uint8 = get_random_number<uint8_t>();
+    gjs_arg_set(&arg, random_uint8);
+    assert_equal(arg.v_uint8, random_uint8);
+    assert_equal(gjs_arg_get<uint8_t>(&arg), random_uint8);
+
+    int16_t random_int16 = get_random_number<int16_t>();
+    gjs_arg_set(&arg, random_int16);
+    assert_equal(arg.v_int16, random_int16);
+    assert_equal(gjs_arg_get<int16_t>(&arg), random_int16);
+
+    uint16_t random_uint16 = get_random_number<uint16_t>();
+    gjs_arg_set(&arg, random_uint16);
+    assert_equal(arg.v_uint16, random_uint16);
+    assert_equal(gjs_arg_get<uint16_t>(&arg), random_uint16);
+
+    int32_t random_int32 = get_random_number<int32_t>();
+    gjs_arg_set(&arg, random_int32);
+    assert_equal(arg.v_int32, random_int32);
+    assert_equal(gjs_arg_get<int32_t>(&arg), random_int32);
+
+    uint32_t random_uint32 = get_random_number<uint32_t>();
+    gjs_arg_set(&arg, random_uint32);
+    assert_equal(arg.v_uint32, random_uint32);
+    assert_equal(gjs_arg_get<uint32_t>(&arg), random_uint32);
+
+    int64_t random_int64 = get_random_number<int64_t>();
+    gjs_arg_set(&arg, random_int64);
+    assert_equal(arg.v_int64, random_int64);
+    assert_equal(gjs_arg_get<int64_t>(&arg), random_int64);
+
+    uint64_t random_uint64 = get_random_number<uint64_t>();
+    gjs_arg_set(&arg, random_uint64);
+    assert_equal(arg.v_uint64, random_uint64);
+    assert_equal(gjs_arg_get<uint64_t>(&arg), random_uint64);
+
+    char32_t random_char32 = get_random_number<char32_t>();
+    gjs_arg_set(&arg, random_char32);
+    assert_equal(static_cast<char32_t>(arg.v_uint32), random_char32);
+    assert_equal(gjs_arg_get<char32_t>(&arg), random_char32);
+
+    float random_float = get_random_number<float>();
+    gjs_arg_set(&arg, random_float);
+    assert_equal(arg.v_float, random_float);
+    assert_equal(gjs_arg_get<float>(&arg), random_float);
+
+    double random_double = get_random_number<double>();
+    gjs_arg_set(&arg, random_double);
+    assert_equal(arg.v_double, random_double);
+    assert_equal(gjs_arg_get<double>(&arg), random_double);
+
+    void* random_ptr = get_random_number<void*>();
+    gjs_arg_set(&arg, random_ptr);
+    assert_equal(arg.v_pointer, random_ptr);
+    assert_equal(gjs_arg_get<void*>(&arg), random_ptr);
+
+    GjsAutoChar cstr = g_strdup("Gjs argument string");
+    gjs_arg_set(&arg, cstr.get());
+    assert_equal(arg.v_string, const_cast<char*>("Gjs argument string"));
+    assert_equal(static_cast<void*>(arg.v_string),
+                 static_cast<void*>(cstr.get()));
+
+    gjs_arg_set<gboolean, GI_TYPE_TAG_BOOLEAN>(&arg, TRUE);
+    g_assert_true(arg.v_boolean);
+    g_assert_true((gjs_arg_get<gboolean, GI_TYPE_TAG_BOOLEAN>(&arg)));
+
+    gjs_arg_set<gboolean, GI_TYPE_TAG_BOOLEAN>(&arg, FALSE);
+    g_assert_false(arg.v_boolean);
+    g_assert_false((gjs_arg_get<gboolean, GI_TYPE_TAG_BOOLEAN>(&arg)));
+
+    gjs_arg_set<gboolean, GI_TYPE_TAG_BOOLEAN>(&arg, TRUE);
+    g_assert_true(arg.v_boolean);
+    gjs_arg_unset<gboolean, GI_TYPE_TAG_BOOLEAN>(&arg);
+    g_assert_false(arg.v_boolean);
+
+    GType random_gtype = get_random_number<GType>();
+    gjs_arg_set<GType, GI_TYPE_TAG_GTYPE>(&arg, random_gtype);
+    if constexpr (std::is_same_v<GType, gsize>)
+        assert_equal(static_cast<GType>(arg.v_size), random_gtype);
+    else if constexpr (std::is_same_v<GType, gulong>)
+        assert_equal(static_cast<GType>(arg.v_ulong), random_gtype);
+    assert_equal(gjs_arg_get<GType, GI_TYPE_TAG_GTYPE>(&arg), random_gtype);
+
+    int random_signed_iface = get_random_number<int>();
+    gjs_arg_set<int, GI_TYPE_TAG_INTERFACE>(&arg, random_signed_iface);
+    assert_equal(arg.v_int, random_signed_iface);
+    assert_equal(gjs_arg_get<int, GI_TYPE_TAG_INTERFACE>(&arg),
+                 random_signed_iface);
+
+    unsigned random_unsigned_iface = get_random_number<unsigned>();
+    gjs_arg_set<unsigned, GI_TYPE_TAG_INTERFACE>(&arg, random_unsigned_iface);
+    assert_equal(arg.v_uint, random_unsigned_iface);
+    assert_equal(gjs_arg_get<unsigned, GI_TYPE_TAG_INTERFACE>(&arg),
+                 random_unsigned_iface);
+}
+
+static void gjstest_test_args_rounded_values() {
+    GIArgument arg = {0};
+
+    gjs_arg_set<int64_t>(&arg, std::numeric_limits<int64_t>::max());
+    g_test_expect_message(
+        G_LOG_DOMAIN, G_LOG_LEVEL_WARNING,
+        "*cannot be safely stored in a JS Number and may be rounded");
+    assert_equal(gjs_arg_get_maybe_rounded<int64_t>(&arg),
+                 static_cast<double>(gjs_arg_get<int64_t>(&arg)));
+    g_test_assert_expected_messages();
+
+    gjs_arg_set<int64_t>(&arg, std::numeric_limits<int64_t>::min());
+    g_test_expect_message(
+        G_LOG_DOMAIN, G_LOG_LEVEL_WARNING,
+        "*cannot be safely stored in a JS Number and may be rounded");
+    assert_equal(gjs_arg_get_maybe_rounded<int64_t>(&arg),
+                 static_cast<double>(gjs_arg_get<int64_t>(&arg)));
+    g_test_assert_expected_messages();
+
+    gjs_arg_set<uint64_t>(&arg, std::numeric_limits<uint64_t>::max());
+    g_test_expect_message(
+        G_LOG_DOMAIN, G_LOG_LEVEL_WARNING,
+        "*cannot be safely stored in a JS Number and may be rounded");
+    assert_equal(gjs_arg_get_maybe_rounded<uint64_t>(&arg),
+                 static_cast<double>(gjs_arg_get<uint64_t>(&arg)));
+    g_test_assert_expected_messages();
+
+    gjs_arg_set<uint64_t>(&arg, std::numeric_limits<uint64_t>::min());
+    assert_equal(gjs_arg_get_maybe_rounded<uint64_t>(&arg), 0.0);
+}
+
 int
 main(int    argc,
      char **argv)
@@ -433,7 +595,24 @@ main(int    argc,
     g_unsetenv("GJS_ENABLE_PROFILER");
     g_unsetenv("GJS_TRACE_FD");
 
-    g_test_init(&argc, &argv, NULL);
+    for (int i = 0; i < argc; i++) {
+        const char* seed = nullptr;
+
+        if (g_str_has_prefix(argv[i], "--cpp-seed=") && strlen(argv[i]) > 11)
+            seed = argv[i] + 11;
+        else if (i < argc - 1 && g_str_equal(argv[i], "--cpp-seed"))
+            seed = argv[i + 1];
+
+        if (seed)
+            cpp_random_seed = std::stoi(seed);
+    }
+
+    g_test_init(&argc, &argv, nullptr);
+
+    if (!cpp_random_seed)
+        cpp_random_seed = g_test_rand_int();
+
+    g_message("Using C++ random seed %u\n", cpp_random_seed);
 
     g_test_add_func("/gjs/context/construct/destroy", gjstest_test_func_gjs_context_construct_destroy);
     g_test_add_func("/gjs/context/construct/eval", gjstest_test_func_gjs_context_construct_eval);
@@ -449,6 +628,10 @@ main(int    argc,
     g_test_add_func("/util/misc/strv/concat/pointers",
                     gjstest_test_func_util_misc_strv_concat_pointers);
 
+    g_test_add_func("/gi/args/set-get-unset", gjstest_test_args_set_get_unset);
+    g_test_add_func("/gi/args/rounded_values",
+                    gjstest_test_args_rounded_values);
+
 #define ADD_JSAPI_UTIL_TEST(path, func)                            \
     g_test_add("/gjs/jsapi/util/" path, GjsUnitTestFixture, NULL,  \
                gjs_unit_test_fixture_setup, func,                  \
@@ -481,6 +664,7 @@ main(int    argc,
     gjs_test_add_tests_for_coverage ();
     gjs_test_add_tests_for_parse_call_args();
     gjs_test_add_tests_for_rooting();
+    gjs_test_add_tests_for_jsapi_utils();
 
     g_test_run();
 
