@@ -1,4 +1,6 @@
 #!/bin/sh
+# SPDX-License-Identifier: MIT OR LGPL-2.0-or-later
+# SPDX-FileCopyrightText: 2019 Philip Chimento <philip.chimento@gmail.com>
 
 SRCDIR=$(pwd)
 
@@ -33,16 +35,17 @@ should_analyze () {
     esac
 }
 
-if ! meson setup _build; then
-    echo 'Meson failed.'
+cd ${BUILDDIR:-_build}
+if ! ninja -t compdb > compile_commands.json; then
+    echo 'Generating compile_commands.json failed.'
     exit 1
 fi
-cd ${BUILDDIR:-_build}
 
 echo "files: $files"
 
 IWYU="python3 $(which iwyu_tool) -p ."
-IWYU_RAW="include-what-you-use -xc++ -std=c++17"
+IWYU_RAW="include-what-you-use -xc++ -std=c++17 -Xiwyu --keep=config.h"
+IWYU_RAW_INC="-I. -I.. $(pkg-config --cflags gobject-introspection-1.0 mozjs-78)"
 PRIVATE_MAPPING="-Xiwyu --mapping_file=$SRCDIR/tools/gjs-private-iwyu.imp -Xiwyu --keep=config.h"
 PUBLIC_MAPPING="-Xiwyu --mapping_file=$SRCDIR/tools/gjs-public-iwyu.imp"
 POSTPROCESS="python3 $SRCDIR/tools/process_iwyu.py"
@@ -50,22 +53,28 @@ EXIT=0
 
 # inline-only files with no implementation file don't appear in the compilation
 # database, so iwyu_tool cannot process them
-if should_analyze $SRCDIR/gi/utils-inl.h; then
-    if ! $IWYU_RAW $(realpath --relative-to=. $SRCDIR/gi/utils-inl.h) 2>&1 \
-        | $POSTPROCESS; then
-        EXIT=1
+for FILE in $SRCDIR/gi/arg-types-inl.h $SRCDIR/gi/js-value-inl.h \
+    $SRCDIR/gi/utils-inl.h $SRCDIR/gjs/enum-utils.h \
+    $SRCDIR/gjs/jsapi-util-args.h $SRCDIR/gjs/jsapi-util-root.h \
+    $SRCDIR/modules/cairo-module.h
+do
+    if should_analyze $FILE; then
+        if ! $IWYU_RAW $(realpath --relative-to=. $FILE) $IWYU_RAW_INC 2>&1 \
+            | $POSTPROCESS; then
+            EXIT=1
+        fi
     fi
-fi
+done
 
 for FILE in $SRCDIR/gi/*.cpp $SRCDIR/gjs/atoms.cpp $SRCDIR/gjs/byteArray.cpp \
     $SRCDIR/gjs/coverage.cpp $SRCDIR/gjs/debugger.cpp \
     $SRCDIR/gjs/deprecation.cpp $SRCDIR/gjs/error-types.cpp \
     $SRCDIR/gjs/engine.cpp $SRCDIR/gjs/global.cpp $SRCDIR/gjs/importer.cpp \
-    $SRCDIR/gjs/jsapi-util-error.cpp $SRCDIR/gjs/jsapi-util-string.cpp \
-    $SRCDIR/gjs/module.cpp $SRCDIR/gjs/native.cpp $SRCDIR/gjs/stack.cpp \
-    $SRCDIR/modules/cairo-*.cpp $SRCDIR/modules/console.cpp \
-    $SRCDIR/modules/print.cpp $SRCDIR/modules/system.cpp $SRCDIR/test/*.cpp \
-    $SRCDIR/util/*.cpp $SRCDIR/libgjs-private/*.c
+    $SRCDIR/gjs/jsapi-util*.cpp $SRCDIR/gjs/module.cpp $SRCDIR/gjs/native.cpp \
+    $SRCDIR/gjs/stack.cpp $SRCDIR/modules/cairo-*.cpp \
+    $SRCDIR/modules/console.cpp $SRCDIR/modules/print.cpp \
+    $SRCDIR/modules/system.cpp $SRCDIR/test/*.cpp $SRCDIR/util/*.cpp \
+    $SRCDIR/libgjs-private/*.c
 do
     if should_analyze $FILE; then
         if ! $IWYU $FILE -- $PRIVATE_MAPPING | $POSTPROCESS; then
@@ -74,13 +83,7 @@ do
     fi
 done
 
-if should_analyze $SRCDIR/gjs/context.cpp; then
-    if ! $IWYU $SRCDIR/gjs/context.cpp -- $PRIVATE_MAPPING \
-        -Xiwyu --check_also=*/gjs/context-private.h | $POSTPROCESS; then
-        EXIT=1
-    fi
-fi
-
+# this header file is named differently from its corresponding implementation
 if ( should_analyze $SRCDIR/gjs/jsapi-dynamic-class.cpp || \
     should_analyze $SRCDIR/gjs/jsapi-class.h ); then
     if ! $IWYU $SRCDIR/gjs/jsapi-dynamic-class.cpp -- $PRIVATE_MAPPING \
@@ -89,38 +92,15 @@ if ( should_analyze $SRCDIR/gjs/jsapi-dynamic-class.cpp || \
     fi
 fi
 
-if ( should_analyze $SRCDIR/gjs/jsapi-util.cpp ||
-    should_analyze $SRCDIR/gjs/jsapi-util-args.h || \
-    should_analyze $SRCDIR/gjs/jsapi-util-root.h ); then
-    if ! $IWYU $SRCDIR/gjs/jsapi-util.cpp -- $PRIVATE_MAPPING \
-        -Xiwyu --check_also=*/gjs/jsapi-util-args.h \
-        -Xiwyu --check_also=*/gjs/jsapi-util-root.h | $POSTPROCESS; then
-        EXIT=1
+# include header files with private implementation along with their main files
+for STEM in gjs/context gjs/mem gjs/profiler modules/cairo; do
+    if should_analyze $SRCDIR/$STEM.cpp; then
+        if ! $IWYU $SRCDIR/$STEM.cpp -- $PRIVATE_MAPPING \
+            -Xiwyu --check_also=*/$STEM-private.h | $POSTPROCESS; then
+            EXIT=1
+        fi
     fi
-fi
-
-if should_analyze $SRCDIR/gjs/mem.cpp; then
-    if ! $IWYU $SRCDIR/gjs/mem.cpp -- $PRIVATE_MAPPING \
-        -Xiwyu --check_also=*/gjs/mem-private.h | $POSTPROCESS; then
-        EXIT=1
-    fi
-fi
-
-if should_analyze $SRCDIR/gjs/profiler.cpp; then
-    if ! $IWYU $SRCDIR/gjs/profiler.cpp -- $PRIVATE_MAPPING \
-        -Xiwyu --check_also=*/gjs/profiler-private.h | $POSTPROCESS; then
-        EXIT=1
-    fi
-fi
-
-if ( should_analyze $SRCDIR/modules/cairo.cpp ||
-    should_analyze $SRCDIR/modules/cairo-module.h ); then
-    if ! $IWYU $SRCDIR/modules/cairo.cpp -- $PRIVATE_MAPPING \
-        -Xiwyu --check_also=*/modules/cairo-module.h \
-        -Xiwyu --check_also=*/modules/cairo-private.h | $POSTPROCESS; then
-        EXIT=1
-    fi
-fi
+done
 
 for FILE in $SRCDIR/gjs/console.cpp $SRCDIR/installed-tests/minijasmine.cpp
 do
